package picontrol;

// Imports
import java.util.ArrayList;

/**
 * Client This class represent a Raspberry working as client
 *
 *
 */
public class Client extends Server {

    // Declarations
    private int numServers; // Number of expected servers on the net
    private ArrayList<Server> listServers; // Array with the servers
    private Status status; // Object with the system status
    private Automatic automatic; // Object with the automata configuration
    private int countErrors; // Counter of communication errors

    // Configuration values
    private int configRetries; // Number of configRetries to send a message
    private int configNumErrorsBeforeEmail; // Numbers of errors before send warning mail.
    private boolean configSendEmailOnError; // Send mail on error
    private int configTemperatureCounter; // Number of temperatures to get the temperature average.
    private int configTimeoutConnectValue; // Time for wait connection before timeout error.
    private int configTimeoutReadValue; // Time for wait a response before timeout error.
    private int configTimeWait; // Time for wait between connections.
    private int configTimeOnlineWait; // Time for wait between connections when a server is offline.
    private double configOptimalTemp; // Optimal temperature of system working.

    /* 
     * Variables for calculate the average of temperatures 
     */
    private int countTemp; // Counter of number of temperatures
    private double sumTempSolar; // Sum of solar temperatures
    private double sumTempExt; // Sum of exterior temperatures

    /**
     * Client(): constructor for class Client.
     */
    public Client() {
        listServers = new ArrayList<>();
        status = new Status();
        countErrors = 0;
        countTemp = 0;
        sumTempSolar = 0;
        sumTempExt = 0;
    }

    // Methods
    /**
     * initStatus: This class set the initial status of the client
     */
    public void initStatus() {
        // Declarations
        Processor proc; // We add the client. They are online

        // Initializations
        proc = new Processor(getIdentifier(), getIpAddress(), true);
        // And normally, we want it working
        proc.setType("Client");

        // Usually, the client don't have any device
        status.addProcessorList(proc);
        status.incNumProcessors();
        status.updateStatusTime();
    }

    /**
     * genNumServers: return the number of servers in the network.
     *
     * @return the numServers
     */
    public int getNumServers() {
        return numServers;
    }

    /**
     * setNumServers: Set the number of servers in the network.
     *
     * @param numServers the numServers to set
     */
    public void setNumServers(int numServers) {
        this.numServers = numServers;
    }

    /**
     * getListServers: return the list of servers in the network.
     *
     * @return the listServers
     */
    public ArrayList<Server> getListServers() {
        return listServers;
    }

    /**
     * setListServers: set the array list with the servers in the network.
     *
     * @param listServers the listServers to set
     */
    public void setListServers(ArrayList<Server> listServers) {
        this.listServers = listServers;
    }

    /**
     * addServer: add a server to the array list of servers.
     *
     * @param server server object to add
     */
    public void addServer(Server server) {
        listServers.add(server);
    }

    /**
     * updateServer: This method update the data of devices connected to a RPi.
     * It also updates the global status of the system.
     *
     * @param server server object to update
     * @param pos    position in the list
     */
    public void updateServer(Server server, int pos) {
        // Declarations
        int num;
        // Initializations
        num = listServers.size();
        if (pos >= num) {
            listServers.add(server);
        } else {
            listServers.set(pos, server);
        }
        // Now, update the status, only if the Rpi is online
        if (server.getOnline()) {
            Processor processor;
            Peripheral peripheral;
            // For each server
            processor = status.getProcessor(server.getIdentifier());
            if (processor == null) {
                // Create the processor
                processor = new Processor(server.getIdentifier(), server.getIpAddress(), true);
                // This is a server
                processor.setType("Server");
                status.addProcessorList(processor);
            }
            processor.setOnline(server.getOnline());
            // And for their devices
            for (int j = 0; j < server.getNumDevices(); j++) {
                peripheral = processor.getPeripheral(j);
                if (peripheral == null) {
                    peripheral = new Peripheral(server.getDevice(j).getName(), server.getDevice(j).getMode());
                    peripheral.setLogicName(server.getDevice(j).getParameter4());
                    processor.addPeripheral(peripheral);
                }
                peripheral.setValue(server.getDevice(j).getValue());
                processor.updatePeripheral(peripheral, j);
            }
            // Update processor with peripherals
            status.updateProcessorList(processor, (pos + 1)); // The first position is for the client
        }
    }

    /**
     * getServer: return the server in the specified position
     *
     * @param pos position in the list of servers
     *
     * @return the server in the specified position
     */
    public Server getServer(int pos) {
        if (getNumServers() < pos) {
            return null;
        }
        return listServers.get(pos);
    }

    /**
     * getServer: return the server whit the specified name
     *
     * @param serverName name of the server to locate.
     *
     * @return Server with the specified serverName
     */
    public Server getServer(String serverName) {
        for (Server server : listServers) {
            if (server.getIdentifier().equals(serverName)) {
                return server;
            }
        }
        return null;
    }

    /**
     * getServerPos: return the position in the array list of servers, of server whit the specified name.
     *
     * @param serverName name of the server to search
     *
     * @return position of server with specified serverName in the list of servers.
     */
    public int getServerPos(String serverName) {
        for (int i = 0; i < listServers.size(); i++) {
            if (getServer(i).getIdentifier().equals(serverName)) {
                return i;
            }
        }
        return -1;
    }

    /**
     * getNumServersOnline: return the number of servers that are online.
     *
     * @return the number of servers that are Online.
     */
    public int getNumServersOnline() {
        int i = 0;

        i = listServers.stream().filter((server) -> (server.getOnline())).map((_item) -> 1).reduce(i, Integer::sum);
        return i;
    }

    /**
     * getCountErrors: return the number of counting errors trying connecting with servers.
     *
     * @return the number of errors.
     */
    public int getCountErrors() {
        return countErrors;
    }

    /**
     * setCountErrors: set the number of errors.
     *
     * @param countErrors the countErrors to set.
     */
    public void setCountErrors(int countErrors) {
        this.countErrors = countErrors;
    }

    /**
     * incCountErrors: increment the number of errors.
     */
    public void incCountErrors() {
        this.countErrors++;
    }

    /**
     * getStatus: return the current status
     *
     * @return the status
     */
    public Status getStatus() {
        return status;
    }

    /**
     * setStatus: set the status
     *
     * @param status the status to set
     */
    public void setStatus(Status status) {
        this.status = status;
    }

    /**
     * getAutomatic: return the automatic object with all the rules
     *
     * @return the automatic
     */
    public Automatic getAutomatic() {
        return automatic;
    }

    /**
     * setAutomatic: set the automatic object with all the rules
     *
     * @param automatic the automatic to set
     */
    public void setAutomatic(Automatic automatic) {
        this.automatic = automatic;
    }

    /**
     * getCountTemp: return the number of temperatures counted
     *
     * @return the countTemp
     */
    public int getCountTemp() {
        return countTemp;
    }

    /**
     * setCountTemp: set the counter to a value
     *
     * @param count value to set the counter of temperatures.
     */
    public void setCountTemp(int count) {
        countTemp = count;
    }

    /**
     * incCountTemp: increments the counter
     *
     */
    public void incCountTemp() {
        countTemp++;
    }

    /**
     * getSumTempSolar: return the sum of solar temperatures.
     *
     * @return the sumTempSolar.
     */
    public double getSumTempSolar() {
        return sumTempSolar;
    }

    /**
     * setSumTempSolar: set the sum of solar temperatures to a value.
     *
     * @param sTempSolar Value of sum of temperatures to set.
     */
    public void setSumTempSolar(double sTempSolar) {
        sumTempSolar = sTempSolar;
    }

    /**
     * addTempSolar: add a value to the solar temperature sum.
     *
     * @param tempSolar value of temperature to set.
     */
    public void addTempSolar(double tempSolar) {
        sumTempSolar += tempSolar;
    }

    /**
     * getSumTempExt: get the sum of the exterior temperature
     *
     * @return the sum of temperatures
     */
    public double getSumTempExt() {
        return sumTempExt;
    }

    /**
     * setSumTempExt: set the sum of the exterior temperature to a specified value
     *
     * @param sTempExt value to set
     */
    public void setSumTempExt(double sTempExt) {
        sumTempExt = sTempExt;
    }

    /**
     * addTempExterior: add a value to the exterior temperature sum
     *
     * @param tempExt temperature to add
     */
    public void addTempExt(double tempExt) {
        sumTempExt += tempExt;
    }

    /**
     * getNumErrorsBeforeEmail: return the number of errors that the system must count before sending a warning email.
     *
     * @return the configNumErrorsBeforeEmail
     */
    public int getConfigNumErrorsBeforeEmail() {
        return configNumErrorsBeforeEmail;
    }

    /**
     * setNumErrorsBeforeEmail: set the number of errors that the system must count before sending a warning email.
     *
     * @param configNumErrorsBeforeEmail the configNumErrorsBeforeEmail to set
     */
    public void setConfigNumErrorsBeforeEmail(int configNumErrorsBeforeEmail) {
        this.configNumErrorsBeforeEmail = configNumErrorsBeforeEmail;
    }

    /**
     * getSendEmailOnError: return if we want send a email on error case.
     *
     * @return the configSendEmailOnError
     */
    public boolean getSendEmailOnError() {
        return configSendEmailOnError;
    }

    /**
     * setSendEmailOnError: set if we want send a email on error case.
     *
     * @param configSendEmailOnError the configSendEmailOnError to set
     */
    public void setConfigSendEmailOnError(boolean configSendEmailOnError) {
        this.configSendEmailOnError = configSendEmailOnError;
    }

    /**
     * getTemperatureCounter: get the number of temperatures to obtain the average of temperatures.
     *
     * @return the configTemperatureCounter
     */
    public int getConfigTemperatureCounter() {
        return configTemperatureCounter;
    }

    /**
     * setTemperatureCounter: set the number of temperatures to obtain the average of temperatures.
     *
     * @param configTemperatureCounter the configTemperatureCounter to set
     */
    public void setConfigTemperatureCounter(int configTemperatureCounter) {
        this.configTemperatureCounter = configTemperatureCounter;
    }

    /**
     * getTimeoutConnectValue: get the time to wait connection before timeout error.
     *
     * @return the configTimeoutConnectValue
     */
    public int getConfigTimeoutConnectValue() {
        return configTimeoutConnectValue;
    }

    /**
     * setTimeoutConnectValue: get the time to wait connection before timeout error.
     *
     * @param configTimeoutConnectValue the configTimeoutConnectValue to set
     */
    public void setConfigTimeoutConnectValue(int configTimeoutConnectValue) {
        this.configTimeoutConnectValue = configTimeoutConnectValue;
    }

    /**
     * getTimeoutReadValue: get the time for wait a response before timeout error.
     *
     * @return the configTimeoutReadValue
     */
    public int getConfigTimeoutReadValue() {
        return configTimeoutReadValue;
    }

    /**
     * setTimeoutReadValue: set the time for wait a response before timeout error.
     *
     * @param configTimeoutReadValue the configTimeoutReadValue to set
     */
    public void setConfigTimeoutReadValue(int configTimeoutReadValue) {
        this.configTimeoutReadValue = configTimeoutReadValue;
    }

    /**
     * getTimeWait: get the time for wait between connections.
     *
     * @return the configTimeWait
     */
    public int getConfigTimeWait() {
        return configTimeWait;
    }

    /**
     * setTimeWait: set the time for wait between connections.
     *
     * @param configTimeWait the configTimeWait to set
     */
    public void setConfigTimeWait(int configTimeWait) {
        this.configTimeWait = configTimeWait;
    }

    /**
     * getTimeOnlineWait: get the time for wait between connections when a server is offline.
     *
     * @return the configTimeOnlineWait
     */
    public int getConfigTimeOnlineWait() {
        return configTimeOnlineWait;
    }

    /**
     * setTimeOnlineWait: set the time for wait between connections when a server is offline.
     *
     * @param configTimeOnlineWait the configTimeOnlineWait to set
     */
    public void setConfigTimeOnlineWait(int configTimeOnlineWait) {
        this.configTimeOnlineWait = configTimeOnlineWait;
    }

    /**
     * getConfigOptimalTemp: get the optimal temperature for the system.
     *
     * @return the configOptimalTemp
     */
    public double getConfigOptimalTemp() {
        return configOptimalTemp;
    }

    /**
     * setConfigOptimalTemp: set the optimal temperature for the system.
     *
     * @param configOptimalTemp the configOptimalTemp to set
     */
    public void setConfigOptimalTemp(double configOptimalTemp) {
        this.configOptimalTemp = configOptimalTemp;
    }

    /**
     * getRetries: get the number of configRetries to try send the messages.
     *
     * @return the configRetries
     */
    public int getConfigRetries() {
        return configRetries;
    }

    /**
     * @param configRetries the configRetries to set
     */
    public void setConfigRetries(int configRetries) {
        this.configRetries = configRetries;
    }

    /**
     * toString: return the string with the client information
     *
     * @return string with the client information
     */
    @Override
    public String toString() {
        Server server;
        StringBuilder str = new StringBuilder();
        str.append("Client:\n").append("-++++++++++++++++++++++++++++++++++++++++++\n");
        int i;

        for (i = 0; i < listServers.size(); i++) {
            str.append("Server ").append((i + 1)).append("\n")
                    .append(".............................................\n");
            server = (Server) getServer(i);
            if (server.getOnline()) {
                str.append("Identifier:     ").append(server.getIdentifier()).append("\n");
                str.append("LAN Address:    ").append(server.getIpAddress()).append("\n");
                str.append("Port:           ").append(server.getPort()).append("\n");
                str.append("Num Devices:    ").append(server.getNumDevices()).append("\n");
                for (int j = 0; j < server.getNumDevices(); j++) {
                    str.append("Device ").append(j).append("\n");
                    str.append(server.getDevice(j)).append("\n");
                }
            } else {
                str.append("Server Offline!!!\n");
            }
        }
        return str.toString();
    }

}
