package picontrol;

import com.google.gson.Gson;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.SocketTimeoutException;
import java.net.URL;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 *
 * This class implements all communications methods for the client
 *
 *
 */
public final class Communication {

    // Java Log4J logger for logs and messages
    static final Logger logger = LogManager.getLogger(picontrol.Communication.class.getName());

    /**
     * sendGET: This method send a GET request to the url specified.
     *
     * @param client     Object client with all the information.
     * @param urlAddress String with url: http://address/resource
     *
     * @return Object Response
     */
    public static Response sendGET(Client client, String urlAddress) {
        // String with the URL
        URL url;
        // HTTP URL Connection with sever
        HttpURLConnection connectionWithServer;
        // Htpp URL Response code for detect errors
        int responseCode;
        // Buffered Reader for read the input
        BufferedReader in;
        // String with the response
        StringBuilder responseStr;
        // For handle error connections
        Constants.CONNECTION_ERROR error;
        // Object with the response
        Response objResponse;

        // Initialize variables
        objResponse = new Response();
        // By defaul, I suposse no error
        error = Constants.CONNECTION_ERROR.NO_ERROR;
        String line; //
        responseStr = new StringBuilder();
        responseCode = HttpURLConnection.HTTP_OK;
        connectionWithServer = null;
        try {
            url = new URL(urlAddress);
            connectionWithServer = (HttpURLConnection) url.openConnection();
            /*
             * Set type and values of connection. GET is the default method, so this is optional
             */
            connectionWithServer.setRequestMethod("GET");
            connectionWithServer.setRequestProperty("User-Agent", Constants.USER_AGENT);
            connectionWithServer.setConnectTimeout(client.getConfigTimeoutConnectValue());
            connectionWithServer.setReadTimeout(client.getConfigTimeoutReadValue());
            // Header
            responseCode = connectionWithServer.getResponseCode();
        } catch (SocketTimeoutException ste) {
            logger.warn("Timeout connection " + ste.getMessage() + " with " + urlAddress);
            error = Constants.CONNECTION_ERROR.TIMEOUT_ERROR;
        } catch (IOException ioe) {
            logger.warn("Error!! " + ioe.getMessage() + " with " + urlAddress);
            error = Constants.CONNECTION_ERROR.IO_ERROR;
        }
        if (responseCode == HttpURLConnection.HTTP_OK) {
            try {
                in = new BufferedReader(new InputStreamReader(connectionWithServer.getInputStream()));
                // Waiting for read the response
                while ((line = in.readLine()) != null) {
                    responseStr.append(line);
                }
                in.close();
            } catch (IOException ioe) {
                logger.warn("Error waiting for the response!! \n" + ioe.getMessage());
                error = Constants.CONNECTION_ERROR.IO_ERROR;
            }
        } else {
            // Error
            logger.warn("GET request error with " + urlAddress);
            responseStr = new StringBuilder("GET request error!!!");
            error = Constants.CONNECTION_ERROR.GET_ERROR;
        }
        objResponse.setMessage(responseStr.toString());
        objResponse.setErrorCode(error);
        return objResponse;
    }

    /**
     * sendGET: This method send a GET request with the order to the server specified.
     *
     * @param client Object client with all the info
     * @param server Destination server
     * @param order  Message to send
     *
     * @return true if there has been no error in the transmission
     */
    public static boolean sendGET(Client client, Server server, String order) {
        // Declaration
        String urlAddress;
        Response objResponse;
        // Initialization
        urlAddress = Utilities.getAddress(server);
        urlAddress = urlAddress + "/" + order;
        // Send the request to the url 
        objResponse = sendGET(client, urlAddress);
        return (objResponse.getErrorCode() == Constants.CONNECTION_ERROR.NO_ERROR);
    }

    /**
     * sendPOST: This method send a POST request with the command + parameters to the server specified.
     *
     * @param client       Object client with all the info
     * @param strServer    Name of the destination server
     * @param strCommand   Command to send to the server
     * @param strParameter Parameters to send to the server
     *
     * @return a object Response with the response.
     */
    public static Response sendPOST(Client client, String strServer, String strCommand, String strParameter) {
        // Declarations
        Request objRequest; // Object with the request
        Response objResponse; // Object with the response
        Server server; // Destination server 
        String address; // String with the URL of the server

        // Initialization
        server = client.getServer(strServer);
        objResponse = new Response(Constants.CONNECTION_ERROR.NO_ERROR);
        if (server == null) {
            logger.fatal("Error getting the Server name!!. Bad name in config file?");
            System.exit(-1);
        }
        if (testOnline(client, server)) {
            /*
             * If the server is online, update on client list only, if it isn't on the list
             */
            server.setOnline(true);
            address = Utilities.getAddress(server);
            objRequest = new Request(strCommand, strParameter);
            objResponse = sendPOST(client, address, objRequest);
        } else {
            /*
             * Server isn't on line.
             */
            logger.warn("Server " + strServer + " is Offline!!");
            server.setOnline(false);
        }
        return objResponse;
    }

    /**
     * sendPOST: This method send through POST, a Request object to server specified by URL
     *
     * @param client     Object client with all the info
     * @param urlAddress URL Address of the server
     * @param objRequest Object with the request.
     *
     * @return a object Response with the response.
     */
    public static Response sendPOST(Client client, String urlAddress, Request objRequest) {
        // Declarations
        URL url; // Object URL with the address of the server
        HttpURLConnection connectionWithServer; // Http Connection with the server
        Response objResponse; // Object with the response
        Gson jsonObj; // Object to serialize in JSON format
        Constants.CONNECTION_ERROR errorCode; // Error code of the connection
        int responseCode; // Code of the response
        BufferedReader in; // Stream buffer for read
        String line; // Line for read in the stream
        String strRequest; // String with the serialized Request
        OutputStream writeStream; // Stream to write

        // Initializations
        objResponse = new Response();
        jsonObj = new Gson();
        errorCode = Constants.CONNECTION_ERROR.NO_ERROR; // By default, no errors

        // Translate the Request object to a JSON string.
        strRequest = jsonObj.toJson(objRequest);
        try {
            url = new URL(urlAddress);
            connectionWithServer = (HttpURLConnection) url.openConnection();
            // Set type of connection to POST
            connectionWithServer.setRequestMethod("POST");
            // Set timeout values
            connectionWithServer.setConnectTimeout(client.getConfigTimeoutConnectValue());
            connectionWithServer.setReadTimeout(client.getConfigTimeoutReadValue());
            // Add request header
            connectionWithServer.setRequestProperty("User-Agent", Constants.USER_AGENT);
            // --------------
            // POST - START
            connectionWithServer.setDoOutput(true);
            writeStream = connectionWithServer.getOutputStream();
            // Sending the JSON string with the request
            writeStream.write(strRequest.getBytes());
            writeStream.flush();
            writeStream.close();
            // POST - STOP
            // --------------
            // Read the reply
            try {
                // Header
                responseCode = connectionWithServer.getResponseCode();
                logger.trace("Response code: " + responseCode);
                if (responseCode == HttpURLConnection.HTTP_OK) {
                    in = new BufferedReader(new InputStreamReader(connectionWithServer.getInputStream()));
                    // Waiting for read the response
                    StringBuilder sbResponse = new StringBuilder();
                    while ((line = in.readLine()) != null) {
                        sbResponse.append(line);
                    }
                    in.close();
                    // Result
                    objResponse.setMessage(sbResponse.toString());
                } else {
                    logger.warn("POST request error!!!");
                    errorCode = Constants.CONNECTION_ERROR.POST_ERROR;
                }
            } catch (IOException ioe) {
                logger.warn("I/O Error " + ioe.getMessage());
                errorCode = Constants.CONNECTION_ERROR.IO_ERROR;
            }
        } catch (SocketTimeoutException ste) {
            logger.warn("Timeout connection " + ste.getMessage());
            errorCode = Constants.CONNECTION_ERROR.TIMEOUT_ERROR;
        } catch (IOException ioe) {
            logger.warn("Error!! " + ioe.getMessage());
            errorCode = Constants.CONNECTION_ERROR.IO_ERROR;
        }
        objResponse.setErrorCode(errorCode);
        return objResponse;
    }

    /**
     * testOnLine: Test if server number "i" are online.
     *
     * @param client Client with all the information.
     * @param i      Number of server to test
     *
     * @return True or false if the server are online
     */
    public static boolean testOnline(Client client, int i) {
        // Declarations
        Server server; // Server to test

        server = client.getServer(i);
        if (server == null) {
            return false;
        } else {
            return testOnline(client, server);
        }
    }

    /**
     * testOnLine: Check if the server specified are online
     *
     * @param client Client with all the information.
     * @param server Server to test
     *
     * @return True or false if the server are online
     */
    public static boolean testOnline(Client client, Server server) {
        // Declarations
        Response objResponse; // Object with the response
        String urlAddress; // Address of Pi Server

        // Initializations
        urlAddress = Utilities.getAddress(server);
        // Send online command and wait for response
        objResponse = sendGET(client, urlAddress + "/ONLINE");
        // Test if server reply or other error?
        if (Constants.CONNECTION_ERROR.NO_ERROR == (Constants.CONNECTION_ERROR) objResponse.getErrorCode()) {
            return true;
        } else {
            logger.warn("Error trying test online server " + urlAddress + "\n" + objResponse.getMessage());
            return false;
        }
    }

    /**
     * waitForAllServersOnline: wait for all servers go online.
     * If not all servers are online, it send a warning email.
     * One the first time, and every 7 hours
     *
     * @param client client with all the info.
     */
    public static void waitForAllServersOnline(Client client) {
        // Declarations
        Boolean allOnline; // Boolean to indicate if all servers are online
        Response objResponse; // Object with the response
        String strResponse; // String with the serilized response
        Request objRequest; // Object with the request
        Gson jsonObj; // Object for serialized in JSON
        String address; // String with the address of server
        Server server; // Server object
        
    // Variables to determine number of errors and to control the number of warning email sent
        String strEmail; // Text of the body of the email.
        int countEmail; // Only for count the number of email sent
        int countErrors; // Counter errors

        // Initializations
        jsonObj = new Gson();
        objResponse = new Response();
        objRequest = new Request();
        allOnline = false;
        countEmail = 0;
        countErrors = 0;

        // Wait for all servers are online
        while (!allOnline) {
            // Wait a time so the servers go online!!!
            logger.trace("Waiting to servers go online!!");
            try {
                Thread.sleep(client.getConfigTimeOnlineWait());
            } catch (InterruptedException ex) {
                logger.warn("Error waiting!!! " + ex.getMessage());
            }
            for (int i = 0; i < client.getNumServers(); i++) {
                server = client.getServer(i);
                if ((testOnline(client, server))) {
                    // If the server is online, update on client list, but only if it isn't on the list
                    if (!server.getOnline()) {
                        // Get the server's adress
                        address = Utilities.getAddress(server);
                        // Send the command "Config Server" to get the current configuration of the server
                        objRequest.setCommand("CONFIG");
                        objRequest.setParameters("SERVER");
                        objResponse = sendPOST(client, address, objRequest);
                        // Get errors?
                        if (objResponse.getErrorCode() == Constants.CONNECTION_ERROR.NO_ERROR) {
                            strResponse = objResponse.getMessage();
                            objResponse = jsonObj.fromJson(strResponse, Response.class);
                            // Update control client with server data
                            server = jsonObj.fromJson(objResponse.getMessage(), Server.class);
                            client.updateServer(server, i);
                        } else {
                            logger.trace("Error getting response from server!!");
                        }
                    }
                    // The server is online and it's on the list
                } else {
                    // Server isn't on line. 
                    server.setOnline(false);
                }
            } // End for
            logger.trace("Total servers " + client.getNumServers() + "\n Total servers online: " + client.getNumServersOnline());
            // All servers are online?
            if (client.getNumServersOnline() == client.getNumServers()) {
                allOnline = true;
            } else {
                // Process error and send mail, if neccesary
                countErrors++;
                if (countErrors == 1) // it's the first error?
                {
                    countEmail++;
                    strEmail = "Init Solar Pi!!\n"
                            + "Warning number " + countEmail + "\n"
                            + "Some of servers aren't online!!\n";
                    // Check if they are active the send of emails
                    if (client.getSendEmailOnError()) {
                        SendMail.sendMail(strEmail);
                    }
                } else if (countErrors == client.getConfigNumErrorsBeforeEmail()) // it's the error number ?
                {
                    countEmail++;
                    countErrors = 1; // Init the number of erros
                    strEmail = "Init Solar Pi!!\n"
                            + "Warning number " + countEmail + "\n"
                            + "Some of servers aren't online!!\n";
                    // Check if they are active the send of emails
                    if (client.getSendEmailOnError()) {
                        SendMail.sendMail(strEmail);
                    }
                }
            } // If (!all)
        } // End while
    }
}
