package picontrol;

import com.google.gson.Gson;
import com.google.gson.JsonIOException;
import com.google.gson.JsonSyntaxException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Properties;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Processing: this class implement methods for to do some processing
 *
 *
 */
public final class Processing {

    // Logger for logs and messages
    static final Logger logger = LogManager.getLogger(picontrol.Processing.class.getName());

    /**
     * detectMode: This method detect the working mode, depending if exist some
     * manual instruction in the database.
     *
     * @param sqlConnection Connection with the database.
     *
     * @return the mode of working
     */
    public static Constants.WORKING_MODE detectMode(Connection sqlConnection) {
        // Declaration
        Constants.WORKING_MODE mode;
        ResultSet result;
        int count; // Count number of rows on the SQL database with valid manual orders

        count = 0; //
        // By default, automatic mode
        mode = Constants.WORKING_MODE.AUTOMATIC;
        // Get the data from database
        result = SQLUtils.dbQuerySQL(sqlConnection, Constants.SQL_MANUAL_ORDERS_COUNT);
        try {
            result.next();
            count = result.getInt(1);
        } catch (SQLException ex) {
            logger.error("Error detecting working mode from database!! " + ex.getMessage());
        }
        // If we have one of more manual orders, we change to manual
        if (count > 0) {
            mode = Constants.WORKING_MODE.MANUAL;
        }
        return mode;
    }

    /**
     * readManualConfig: Read the manual configuration from the database.
     *
     * @param client        Object client with all the info.
     * @param sqlConnection Connection with the database.
     *
     * @return ArrayList with manual orders to be processed.
     */
    public static ArrayList<ManualOrder> readManualConfig(Client client, Connection sqlConnection) {
        // Declarations
        ResultSet result;
        ArrayList<ManualOrder> listOrders; // List with the manual orders
        ManualOrder order; // Object with the manual order

        // Initialization
        listOrders = new ArrayList<>();

        try {
            result = SQLUtils.dbQuerySQL(sqlConnection, Constants.SQL_MANUAL_ORDERS_GET);
            if (result != null) {
                // For every order
                while (result.next()) {
                    order = new ManualOrder();
                    // We generate the order automatically, so we suposse it are well formed
                    order.setServer(result.getString("server"));
                    order.setOrder(result.getString("order"));
                    order.setType(result.getString("type"));
                    if (result.getString("type").equals("RASPBERRY")) {
                        /* 
                         * Global order
                         *
                         * I mark order as read if it's a global order: SHUTDOWN or REBOOT
                         *
                         * If it's a device order, I execute it, until they are desactived manually
                         */
                        SQLUtils.dbDisableManualOrder(sqlConnection, order);
                    }
                    order.setDevice(result.getString("device"));
                    listOrders.add(order);
                }
            }
        } catch (SQLException ex) {
            logger.error("Error reading manual configuration from database!! " + ex.getMessage());
        }
        return listOrders;
    }

    /**
     * doAutomaticMode: This method do all work to process automatically.
     *
     * @param client Object client with all the info.
     *
     */
    public static void doAutomaticMode(Client client) {
        // Declaration
        Response objResponse; // Response to the action
        Request objRequest; // Request to the server
        Gson jsonObj; // GSON Object
        String strResponse; // String with the response
        String urlAddress; // URL of the server
        Server server; // Server to get or set the action
        Status status; // Status of the system
        String strEmail; // String with the email message
        int countEmail; // Counter of email sent

        // Initialization
        status = client.getStatus();
        countEmail = 0;
        int i, j; // Auxiliary variables

        // Automatic mode
        // ---------------------------
        // Repeat for each server
        for (i = 0; i < client.getNumServers(); i++) {
            server = client.getServer(i);
            // Test if a server online
            if (Communication.testOnline(client, i)) {
                if (!server.getOnline()) {
                    // Server marked as offline. Mark online
                    server.setOnline(true);
                }
                // If it are online, get their current status
                // Server return a Response object in JSON format
                objRequest = new Request("STATUS", "All"); // Status of all devices
                urlAddress = Utilities.getAddress(server);
                for (j = 0; j < client.getConfigRetries(); j++) {
                    objResponse = Communication.sendPOST(client, urlAddress, objRequest);
                    if (objResponse.getErrorCode() == Constants.CONNECTION_ERROR.NO_ERROR) {
                        // The response message is Array of Device status coded in JSON
                        strResponse = objResponse.getMessage();
                        jsonObj = new Gson();
                        try {
                            objResponse = jsonObj.fromJson(strResponse, Response.class);
                            // Update control with server data
                            server = jsonObj.fromJson(objResponse.getMessage(), Server.class);
                        } catch (JsonSyntaxException jse) {
                            logger.warn("Connection broken with server " + server.getIdentifier()
                                    + " at " + server.getIpAddress() + "!!\n" + jse.getMessage());
                        }
                        // exit from the retries loop
                        j = client.getConfigRetries();
                    } else {
                        // A error happened!!!
                        logger.warn("Error in connection with Server " + server.getIdentifier() + " at "
                                + server.getIpAddress());
                        // Mark offline. Used for show info & for determine & update system status
                        server.setOnline(false);
                    } // Error with connection
                } // For retries
            } else {
                // Server isn't online
                logger.warn("Server " + server.getIdentifier() + " at " + server.getIpAddress() + " is OffLine!!");
                // Mark offline: Used to determine & update system status
                server.setOnline(false);
            } // End - Server online
            client.updateServer(server, i);
        } // End for
        // We have all servers online?

        if (client.getNumServersOnline() == client.getNumServers()) {
            client.setCountErrors(0);
            // Update the current status
            status.setDate(Calendar.getInstance().getTime());
            determineAction(client);
        } else {
            // Send a mail by num of errors, about 7 hours
            // I send one when the first error ocurs, and every +-7 hours
            if (client.getCountErrors() == 0) {
                strEmail = "This is the first error on Solar Pi!!\n"
                        + "Some of servers are offline!!\n";
                countEmail++;
                // Check if they are active the send of emails
                if (client.getSendEmailOnError()) {
                    SendMail.sendMail(strEmail);
                }
            }
            client.incCountErrors();
            // I expect about 360 errors for hour.
            if (client.getCountErrors() == client.getConfigNumErrorsBeforeEmail()) {
                client.setCountErrors(1);
                strEmail = "This is the email number " + countEmail + " with warnings about error on Solar Pi!!\n"
                        + "Some of servers are offline!!\n";
                // Check if they are active the send of emails
                if (client.getSendEmailOnError()) {
                    SendMail.sendMail(strEmail);
                }
            }
            logger.warn("Num. Errors: " + client.getCountErrors());
        }
    }

    /**
     * determineAction: this method determine the action to execute according to the state.
     *
     * @param client Object client with all the info.
     */
    public static void determineAction(Client client) {
        // Declaration
        Status status; // System status
        Response objResponse; // Response to the action
        Automatic automatic; // Automatic configuration
        AutomaticState currentState; // Current state
        AutomaticState newState; // Next state
        String strNextState; // Name of the next state
        boolean newStateSelected; // Used for always select the first rule
        String strValue; // Value of status of one device
        String strValueDesired;// Desired value of status of one device
        double temperature; // For the temperature
        double temperatureDesired; // Temperature desired
        int numPeriph, numProc; // For count number  of logical servers and devices
        String command; // Name of the command to send to the servers
        int i, j; // Aux variables
        boolean error;

        // Initialization
        status = client.getStatus(); // Get the current status of servers
        automatic = client.getAutomatic(); // and the desired state implemented
        // Get the current state from the automata
        currentState = automatic.getCurrentState();
        newState = automatic.getCurrentState(); // By default, the new state is the current
        newStateSelected = false;
        error = false;
        strValue = null;
        // Temperature
        temperature = 0.0;
        // Process all rules
        for (AutomaticRule rule : currentState.getListRules()) {
            // Run all the rules accord their type
            if (rule.getRuleType().equals("C")) {
                // We only change of state if the water of termosiphon isn't hot, so
                if (rule.getNameSensor().equals(Constants.SENSOR_TEMP_SOLAR)) {
                    /*
                     * *****************************************************
                     * To get the temperature of water accumulated in the
                     * thermosiphon, I locate the PiServer that has the
                     * sensor with that name connected
                     *****************************************************/
                    numProc = status.getProcessorList().size();
                    for (i = 0; i < numProc; i++) {
                        // We can use:
                        // for (Processor proc: status.getProcessorList), but....
                        Processor proc = status.getProcessor(i);
                        numPeriph = proc.getlistPeripherals().size();
                        for (j = 0; j < numPeriph; j++) {
                            Peripheral perif = proc.getPeripheral(j);
                            if (perif.getName().equals(Constants.SENSOR_TEMP_SOLAR)) {
                                strValue = perif.getValue();
                                temperature = Double.parseDouble(strValue);
                                // i use for(i), so I can end in advance in both loops!!
                                j = numPeriph;
                                i = numProc;
                            }
                        }
                    }
                    strValueDesired = rule.getValue();
                    temperatureDesired = Double.parseDouble(strValueDesired);
                    // Test the temperature to change of state
                    if (temperature > temperatureDesired) {
                        if (!newStateSelected) {
                            strNextState = rule.getNewState();
                            newState = client.getAutomatic().getState(strNextState);
                            newStateSelected = true;
                        }
                    } // End update state
                } else {
                    /*
                     * *****************************************************
                     * This is the temperature of accumulator heated by the
                     * boiler. We only have a interrupt thermostat detected
                     * by SensorA. So we manipulate accordingly
		     * *****************************************************
                     */
                    // Locate server with sensor of aux accumulator, and get their value
                    numProc = status.getProcessorList().size();
                    for (i = 0; i < numProc; i++) {
                        Processor proc = status.getProcessor(i);
                        numPeriph = proc.getlistPeripherals().size();
                        for (j = 0; j < numPeriph; j++) {
                            Peripheral perif = proc.getPeripheral(j);
                            if (perif.getName().equals(Constants.SENSOR_TEMP_AUX)) {
                                strValue = perif.getValue();
                                // We can break!!
                                i = numProc;
                                j = numPeriph;
                            }
                        }
                    }
                    strValueDesired = rule.getValue();
                    if (strValue.equals(strValueDesired)) {
                        // We change the state to the desired
                        if (!newStateSelected) {
                            strNextState = rule.getNewState();
                            newState = client.getAutomatic().getState(strNextState);
                            newStateSelected = true;
                        }
                    }
                } // End of rules C
            } else {
                /*
		 * *************************************************************
                 * Processing rules of type P: we compare the current state of
		 * the actuators and sensors with the desired state in the
                 * automatic and we actuate accordingly
		 * **************************************************************
                 */
                // Read the info of the desired status of the actuator and the server
                String strServer = rule.getServer();
                String typeSensor = rule.getTypeSensor();
                String nameSensor = rule.getNameSensor();
                strValueDesired = rule.getValue();
                // We ignore the Sensors and actuators who state in the rules are indiferent
                if (!strValueDesired.equals("INDIFERENT")) {
                    // Process the status of device
                    command = "";
                    switch (typeSensor) {
                        case "O":
                        case "OUTPUT":
                        case "OUT":
                            switch (strValueDesired) {
                                case "ON":
                                    // Send a ACTIVATE command
                                    command = "ACTIVATE";
                                    break;
                                case "OFF":
                                    // Send a DEACTIVATE command
                                    command = "DEACTIVATE";
                                    break;
                            }
                            break;
                        case "I":
                        case "IN":
                        case "INPUT":
                            // Send a READ command
                            command = "READ";
                            break;
                    }
                    logger.trace("Sending to " + strServer + " the command: " + command + " - " + nameSensor);
                    for (i = 1; i <= client.getConfigRetries(); i++) {
                        objResponse = Communication.sendPOST(client, strServer, command, nameSensor);
                        if (objResponse.getErrorCode() != Constants.CONNECTION_ERROR.NO_ERROR) {
                            error = true;
                            logger.warn("Error sending " + command + " to server " + strServer + "!!!. Try " + i);
                        } else {
                            i = client.getConfigRetries();
                        }
                    }
                }
            } // Type P rules
        } // End of test rules
        // If not error, we stay in the same status....
        if (!error) {
            // Update the new state in the client
            automatic.setCurrentState(newState);
            logger.trace("New state: " + newState.getName());
            client.setAutomatic(automatic);
            // We don't need return the client object...
        }
    }

    /**
     * doManualMode: Prepare the list with the orders indicated in the database.
     *
     * @param client        Object client with all the info.
     * @param sqlConnection Connection with the database.
     *
     * @return boolean to indicate if the client must exit.
     */
    public static boolean doManualMode(Client client, Connection sqlConnection) {
        boolean end;

        // Manual mode
        ArrayList<ManualOrder> listOrders;

        listOrders = readManualConfig(client, sqlConnection);
        end = applyManualMode(client, listOrders, sqlConnection);
        return end;
    }

    /**
     * applyManualMode: Execute the orders specified in manual configuration file.
     *
     * @param client        Object client with all the info.
     * @param listOrders    ArrayList with all manuals orders to be processed.
     * @param sqlConnection Connection with the database.
     *
     * @return boolean to indicate if the client must exit.
     */
    public static boolean applyManualMode(Client client, ArrayList<ManualOrder> listOrders, Connection sqlConnection) {
        // Declarations
        String type; // Type of command
        String strOrder; // String with the command
        String strServer; // String with the name server
        String strDevice; // STring with the device name
        Server server; // Server to send the order
        String command; // Command to send to a server
        Response objResponse; // Response from the server
        String strResponse; // String with the response
        Gson jsonObj; // GSon object
        int i; // Auxiliar variable

        // Initializations
        boolean end = false;
        command = "";
        jsonObj = new Gson();
        if (listOrders != null) {
            for (ManualOrder order : listOrders) {
                type = order.getType();
                strServer = order.getServer();
                strOrder = order.getOrder().toUpperCase();
                switch (type) {
                    case "RASPBERRY":
                        // If the order is for the client??
                        if (strServer.equals(Constants.CLIENT_NAME)) {
                            switch (strOrder) {
                                case "SHUTDOWN":
                                case "REBOOT":
                                    Processing.shutdown(strOrder);
                                    logger.warn(strOrder + " in 30sg!!");
                                    end = true;
                                    break;
                                case "EXIT":
                                    logger.warn("Ending the client!!");
                                    end = true;
                                    break;
                            }
                        } else {
                            // Get the server to send the order
                            server = client.getServer(strServer);
                            // and send the command by GET
                            for (i = 0; i < client.getConfigRetries(); i++) {
                                if (Communication.sendGET(client, server, strOrder)) {
                                    logger.warn("Manual command sent sucessfully!!!");
                                    i = client.getConfigRetries();
                                } else {
                                    logger.warn("Error sending manual command!!! Try " + i);
                                }
                            }
                        }
                        break;
                    // Send the order to specific device
                    case "DEVICE":
                        // Get the server to send the order
                        strDevice = order.getDevice();
                        switch (strOrder) {
                            case "ON":
                                command = "ACTIVATE";
                                break;
                            case "OFF":
                                command = "DEACTIVATE";
                                break;
                            case "AUTO":
                                command = "DEFAULT";
                                // Desactivate the manual order (disable from database of manual orders)
                                SQLUtils.dbDisableManualOrder(sqlConnection, order);
                                break;
                        }
                        logger.trace("Sending to " + strServer + " the command: " + command + " - " + strDevice);
                        for (i = 1; i <= client.getConfigRetries(); i++) {
                            objResponse = Communication.sendPOST(client, strServer, command, strDevice);
                            if (objResponse.getErrorCode() == Constants.CONNECTION_ERROR.NO_ERROR) {
                                strResponse = objResponse.getMessage();
                                try {
                                    objResponse = jsonObj.fromJson(strResponse, Response.class);
                                    if (objResponse != null) {
                                        // Update control client with server data
                                        server = jsonObj.fromJson(objResponse.getMessage(), Server.class);
                                        int pos = client.getServerPos(strServer);
                                        if (pos >= 0) {
                                            client.updateServer(server, pos);
                                            // Exit from the tries loop
                                            i = client.getConfigRetries();
                                        } else {
                                            logger.warn("Error updating server " + strServer + "!!!- Try " + i);
                                        }
                                    } else {
                                        logger.warn("Error in response!!");
                                    }
                                } catch (JsonSyntaxException | JsonIOException jse) {
                                    logger.warn("Error in JSON object!!!");
                                }
                            } else {
                                logger.warn("Error sending " + command + " to server " + strServer + "!!!");
                            }
                        }
                        // Modify the status in the database
                        SQLUtils.dbUpdateStatus(sqlConnection, strServer, strDevice, strOrder);
                        break;
                }
            }
        }
        return end;
    }

    /**
     * initServers: this method create the server objects, making a array of n Server with IP address,
     * etc..
     *
     * In the begin, only with their IP address, port and default server name.
     *
     * @param client Object client with all the info.
     *
     * @return Client with info about servers.
     */
    public static Client initServers(Client client) {
        // Declarations
        int host;
        String network;
        String aux;
        int hostServer;
        Server server;

        // Get from the IP address the host and network
        network = Utilities.getNetwork(client.getIpAddress());
        // Network and host of client
        host = Utilities.getHost(client.getIpAddress());
        for (int i = 0; i < client.getNumServers(); i++) {
            // Initialize ip address & port of servers
            hostServer = host + i + 1;
            aux = network + "." + hostServer;
            // Instantiate new server
            server = new Server();
            // Server IP address
            server.setIpAddress(aux);
            // Server port
            server.setPort(client.getPort());
            // Default Server name
            server.setIdentifier(Constants.SERVER_NAME + (i + 1));
            server.setOnline(false);
            client.addServer(server);
        }
        return client;
    }

    /**
     * readAutomaticConfig: read the file with the automatic configuration.
     *
     * This automata 'automatic' implements the rules to use the system in automatic mode.
     *
     * @param client Object client with all the info.
     *
     * @return Automata with all info read from file.
     */
    public static Automatic readAutomaticConfig(Client client) {
        // Declarations
        int numRules; // Number of rules
        int numStates; // Number of states
        int i, j; // Auxiliary variables
        String strLine; // String with the line read from the file
        String filename; // Name of config file

        // Objects who conform the automatic automata
        Automatic automatic;
        AutomaticState state;
        AutomaticRule rule;

        // Initialization
        automatic = null;
        Properties properties = new Properties();
        filename = Constants.CONFIG_PATH + Constants.CONFIG_AUTOMATIC_FILENAME;
        try {
            InputStream is = new FileInputStream(filename);
            // Load properties file
            properties.load(is);
            // Read properties and set values
            numStates = Integer.parseInt(properties.getProperty("numStatus"));
            automatic = new Automatic(numStates); // Create automatic with
            // numStatus status
            for (i = 1; i <= numStates; i++) {
                strLine = properties.getProperty("name" + i);
                numRules = Integer.parseInt(properties.getProperty("numRulesSt" + i));
                state = new AutomaticState(strLine, numRules);
                for (j = 1; j <= numRules; j++) {
                    // Read Rules
                    strLine = properties.getProperty("St" + i + "Rule" + j);
                    rule = new AutomaticRule();
                    rule.setName("St" + i + "Rule" + j);
                    String[] parameters = strLine.split(",");
                    rule.setRuleType(parameters[0]);
                    rule.setServer(parameters[1]);
                    rule.setTypeSensor(parameters[2]);
                    rule.setNameSensor(parameters[3]);
                    if (parameters[4].equals("TMAX")) {
                        rule.setValue(Double.toString(client.getConfigOptimalTemp()));
                    } else {
                        rule.setValue(parameters[4]);
                    }
                    rule.setNewState(parameters[5]);
                    rule.setComment(parameters[6]);
                    state.addListRules(rule);
                }
                automatic.addListStates(state);
                is.close();
            }
        } catch (FileNotFoundException fnfe) {
            logger.fatal("Error Reading config file: " + Constants.CONFIG_AUTOMATIC_FILENAME + "\n" + fnfe.getMessage());
            System.exit(-1);
        } catch (IOException ioe) {
            logger.fatal("Input/Output Error " + ioe.getMessage());
            System.exit(-1);
        } catch (NumberFormatException e) {
            logger.fatal("Error parsing config " + Constants.CONFIG_AUTOMATIC_FILENAME + "file.");
            System.exit(-1);
        }
        // Indicate the first state in the automatic mode
        state = automatic.getState(Constants.INITIAL_STATE);
        automatic.setCurrentState(state);
        return automatic;
    }

    /**
     * getClientConfig Read the configuration file and update the client.
     *
     * @param client Object Client with all the info.
     *
     * @return Client Object Client with the updated info read from configuration file.
     */
    public static Client getClientConfig(Client client) {
        // Declaration
        Properties properties;
        String filename;
        InputStream is;

        // Initializations
        properties = new Properties();
        // For testing

        filename = Constants.CONFIG_PATH + Constants.CONFIG_CLIENT_FILENAME;
        // Configuration File Name
        try {
            is = new FileInputStream(filename);
            // Load properties file
            properties.load(is);
            // Read properties and set values
            client.setIdentifier(properties.getProperty("identifier"));
            client.setIpAddress(properties.getProperty("address"));
            client.setPort(Integer.parseInt(properties.getProperty("port")));
            client.setNumServers(Integer.parseInt(properties.getProperty("numServers")));
            // Read config values
            client.setConfigRetries(Integer.parseInt(properties.getProperty("retry")));
            client.setConfigNumErrorsBeforeEmail(Integer.parseInt(properties.getProperty("numErrors")));
            client.setConfigSendEmailOnError(Boolean.parseBoolean(properties.getProperty("sendMail")));
            client.setConfigTemperatureCounter(Integer.parseInt(properties.getProperty("tempCounter")));
            client.setConfigTimeoutConnectValue(Integer.parseInt(properties.getProperty("connTimeout")));
            client.setConfigTimeoutReadValue(Integer.parseInt(properties.getProperty("readTimeout")));
            client.setConfigTimeWait(Integer.parseInt(properties.getProperty("waitTime")));
            client.setConfigTimeOnlineWait(Integer.parseInt(properties.getProperty("waitOnline")));
            client.setConfigOptimalTemp(Double.parseDouble(properties.getProperty("optimalTemperature")));
            is.close();
        } catch (FileNotFoundException fnfe) {
            logger.fatal("Error Reading config file: " + filename + "\n" + fnfe.getMessage());
            System.exit(-1);
        } catch (IOException ioe) {
            logger.fatal("Input/Output Error " + ioe.getMessage());
            System.exit(-1);
        } catch (NumberFormatException e) {
            logger.fatal("Error parsing config file " + filename);
            System.exit(-1);
        }
        return client;
    }

    /**
     * updateStatusInServers: this method updates the current status in the servers.
     *
     * @param client Object client with all info about system.
     */
    public static void updateStatusInServers(Client client) {
        Gson jsonObj; // Object for JSON
        Status status; // Current global status
        Response objResponse; // Object with the response
        String strStatus; // String with the JSON serialized status.
        int i;

        status = client.getStatus();
        for (Server srv : client.getListServers()) {
            // Initializations
            jsonObj = new Gson();
            strStatus = jsonObj.toJson((Status) status);
            for (i = 1; i <= client.getConfigRetries(); i++) {
                objResponse = Communication.sendPOST(client, srv.getIdentifier(), "UPDATE", strStatus);
                if (objResponse.getErrorCode() == Constants.CONNECTION_ERROR.NO_ERROR) {
                    String strResponse;
                    strResponse = objResponse.getMessage();
                    logger.trace("Updated global status " + strResponse);
                    // Break for loop.
                    i = client.getConfigRetries();  // <-> break;
                } else {
                    logger.warn("Error updating global status to server " + srv.getIdentifier() + "!!!- Try " + i);
                }
            }
        }
    }

    /**
     * shutdown: This method decide who command to execute: sudo shutdown or
     * sudo reboot, calling to execCommandLinunx whit the correct command.
     *
     * @param opt String with the option to execute.
     */
    public static void shutdown(String opt) {
        String command;

        command = "sudo shutdown ";
        switch (opt) {
            case "SHUTDOWN":
                command += "-h 1";
                break;
            case "REBOOT":
                command += "-r 1";
                break;
        }
        execCommandLinux(command);
    }

    /**
     * execCommandLinux: This method execute a linux command passed as parameter.
     *
     * @param command String with the command to execute.
     */
    public static void execCommandLinux(String command) {
        Runtime runtime;

        runtime = Runtime.getRuntime();
        try {
            runtime.exec(command);
        } catch (IOException ioe) {
            logger.warn("Error trying do " + command + "\n" + ioe.getMessage());
        }
    }
}
