package picontrol;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.Date;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * SQLUtils: Class who implements the SQL methods establishes the connection
 * with the database.
 *
 * 
 */
public final class SQLUtils {

    // Logger for logs and messages
    static final Logger logger = LogManager.getLogger(picontrol.SQLUtils.class.getName());

    /**
     * dbConnect: This method connect with the SQL database.
     *
     * @param conn Connection with database.
     *
     * @return Established connection with database.
     */
    public static Connection dbConnect(Connection conn) {
        // Implementation
        String DSN; // String wiht the DSN of connection

        // Inizialization
        DSN = Constants.DB_DSN + "://" + Constants.DB_HOST + "/"
                + Constants.DB_NAME
                + "?user=" + Constants.DB_USER
                + "&password=" + Constants.DB_PASS;
        try {
            conn = DriverManager.getConnection(DSN);
        } catch (SQLException ex) {
            logger.fatal("Database Error: " + ex.getMessage());
            System.exit(-1);
        }
        return conn;
    }

    /**
     * dbClose: This method close the connection with the database.
     *
     * @param conn Established connection with database.
     *
     */
    public static void dbClose(Connection conn) {
        try {
            conn.close();
        } catch (SQLException ex) {
            logger.fatal("SQL Error: " + ex.getMessage());
            System.exit(-1);
        }
    }

    /**
     * dbQuerySQL: this method execute a SQL query in the specified database connection.
     *
     * @param conn Established connection with database.
     * @param SQL String with the SQL query to execute
     *
     * @return Resultset with the result.
     */
    public static ResultSet dbQuerySQL(Connection conn, String SQL) {
        ResultSet result;
        Statement stmt;

        result = null;
        try {
            stmt = conn.createStatement();
            result = stmt.executeQuery(SQL);
        } catch (SQLException sqle) {
            logger.warn("SQL Error: " + sqle.getMessage());
        }
        return result;
    }

    /**
     * getTimestamp: this method convert a Date() to SQL TimeStamp.
     *
     * @param date date to convert
     *
     * @return Timestamp in SQL format
     */
    public Timestamp getTimestamp(java.util.Date date) {
        return date == null ? null : new java.sql.Timestamp(date.getTime());
    }

    /**
     * dbStoreStatus: this method store the current status in the database.
     *
     * @param client Object client to get the status.
     * @param sqlConnection Established connection with database.
     */
    public static void dbStoreStatus(Client client, Connection sqlConnection) {
        // Check if exists a status in the database
        if (!dbCheckStatus(sqlConnection, Constants.SQL_STATUS_EXISTS)) {
            // If not exist, we insert it
            dbInsertStatus(client, sqlConnection);
        } else {
            // They exists a stored status. Before change it, check if they are in manual mode
            if (dbCheckStatus(sqlConnection, Constants.SQL_MANUAL_ORDERS_COUNT)) {
                // If not, we update the status
                dbUpdateStatus(client, sqlConnection);
            }
        }
    }

    /**
     * dbStoreStatus: this method store the current status in the database.
     *
     * @param client        Object client with all information.
     * @param sqlConnection Established connection with database.
     */
    public static void dbInsertStatus(Client client, Connection sqlConnection) {
        // Declarations
        Processor processor;
        Peripheral peripheral;
        Status status;
        String tempSolar; // Temperature of termosiphon
        String tempExt; // Exterior temperature

        //Initilization
        status = client.getStatus(); // Get the current status
        tempSolar = "";
        tempExt = "";

        try {
            // We get the values of the status to send to database
            for (int i = 0; i < status.getNumProcessors(); i++) {
                processor = status.getProcessor(i);
                // If it's the client, we ignore it and don't save status...
                if (!(processor.getName().equals(client.getIdentifier()))) {
                    for (int j = 0; j < processor.getlistPeripherals().size(); j++) {
                        peripheral = processor.getPeripheral(j);
                        PreparedStatement pstmt = sqlConnection.prepareStatement(Constants.SQL_STATUS_INSERT);
                        pstmt.setString(1, processor.getName()); // Server name
                        pstmt.setString(2, peripheral.getName()); // Device name
                        pstmt.setString(3, peripheral.getMode()); // Device mode
                        pstmt.setString(4, peripheral.getLogicName()); // Device Logic Name
                        pstmt.setString(5, peripheral.getValue()); // Device value
                        pstmt.setString(6, client.getAutomatic().getCurrentState().getName()); // Status name
                        pstmt.executeUpdate(); // Exec SQL update
                        // Now, store the temperature values
                        switch (peripheral.getName()) {
                            case Constants.SENSOR_TEMP_SOLAR:
                                tempSolar = peripheral.getValue();
                                break;
                            case Constants.SENSOR_TEMP_EXT:
                                tempExt = peripheral.getValue();
                                break;
                        }
                    }
                }
            }
            // Now, we store the temperature values in the temperature table
            dbStoreTemperature(client, sqlConnection, tempSolar, tempExt);
        } catch (SQLException ex) {
            logger.warn("Error storing the status in the database!!" + ex.getMessage());
        }
    }

    /**
     * dbUpdateStatus: this method update the current status in the database.
     *
     * @param client        Object client with all information.
     * @param sqlConnection Established connection with database.
     */
    public static void dbUpdateStatus(Client client, Connection sqlConnection) {
        // Declarations
        Processor processor;
        Peripheral peripheral;
        Status status;
        String tempSolar; // Temperature of termosiphon
        String tempExt; // Exterior temperature
        PreparedStatement pstmt;
        ResultSet result;
        String statusName;
        int id; // Idenfifier of SQL row
        int count;

        //Initilization
        status = client.getStatus(); // Get the current status
        tempSolar = "";
        tempExt = "";
        count = 0;
        id = 0;
        statusName = client.getAutomatic().getCurrentState().getName();
        try {
            // First, check it the system is in the same state
            pstmt = sqlConnection.prepareStatement(Constants.SQL_STATUS_VERIFY_NAME);
            pstmt.setString(1, statusName);
            pstmt.executeQuery();
            result = pstmt.executeQuery();
            while (result.next()) {
                count = result.getInt(1);
            }
            if (count > 0) {
                // The system it's in the same state. Update it!!
                // We get the values of the status to send to database
                for (int i = 0; i < status.getNumProcessors(); i++) {
                    processor = status.getProcessor(i);
                    // If it's the client, we ignore it
                    if (!(processor.getName().equals(client.getIdentifier()))) {
                        for (int j = 0; j < processor.getlistPeripherals().size(); j++) {
                            peripheral = processor.getPeripheral(j);
                            pstmt = sqlConnection.prepareStatement(Constants.SQL_STATUS_SEARCH_ID);
                            pstmt.setString(1, processor.getName()); // Server name
                            pstmt.setString(2, peripheral.getName()); // Device name
                            pstmt.executeQuery(); // Exec SQL query
                            result = pstmt.executeQuery();
                            while (result.next()) {
                                id = result.getInt(1);
                            }
                            pstmt = sqlConnection.prepareStatement(Constants.SQL_STATUS_UPDATE);
                            pstmt.setString(1, peripheral.getValue()); // Server name
                            pstmt.setTimestamp(2, getTimeStamp());
                            pstmt.setInt(3, id); // Id of row 
                            pstmt.executeUpdate();
                            // Now, check if it's a temperature value to store
                            switch (peripheral.getName()) {
                                case Constants.SENSOR_TEMP_SOLAR:
                                    tempSolar = peripheral.getValue();
                                    break;
                                case Constants.SENSOR_TEMP_EXT:
                                    tempExt = peripheral.getValue();
                                    break;
                            }
                        }
                    }
                }
                // Now, we store the temperature values in the temperature table
                dbStoreTemperature(client, sqlConnection, tempSolar, tempExt);
            } else {
                // We store the new status. First, we clean the status table
                dbCleanStatus(sqlConnection);
                // And we insert the new status
                dbInsertStatus(client, sqlConnection);
            }
        } catch (SQLException ex) {
            logger.warn("Error updating status on database!! " + ex.getMessage());
        }
    }

    /**
     * dbUpdateStatus: this method update the status row for the specified device of a specified server.
     *
     * @param sqlConnection Established connection with database.
     * @param strServer     Server name.
     * @param strDevice     Device name.
     * @param strOrder      Order to the device.
     */
    public static void dbUpdateStatus(Connection sqlConnection, String strServer, String strDevice, String strOrder) {
        // Declarations
        PreparedStatement pstmt;
        ResultSet result;
        int id;
        // Inicializations
        id = 0;
        try {
            pstmt = sqlConnection.prepareStatement(Constants.SQL_STATUS_SEARCH_ID);
            pstmt.setString(1, strServer); // Server name
            pstmt.setString(2, strDevice); // Device name
            pstmt.executeQuery(); // Exec SQL query
            result = pstmt.executeQuery();
            while (result.next()) {
                id = result.getInt(1);
            }
            pstmt = sqlConnection.prepareStatement(Constants.SQL_STATUS_UPDATE);
            pstmt.setString(1, strOrder); // store strOrder => status of output device
            pstmt.setTimestamp(2, getTimeStamp()); // Update the timestamp
            pstmt.setInt(3, id); // Id of row
            pstmt.executeUpdate();
        } catch (SQLException se) {
            logger.warn("Error updating status on database!! " + se.getMessage());
        }
    }

    /**
     * dbCheckStatus: this method check if they exist a status stored in the database,
     * or if the current status stored in the database has a manual order.
     *
     * @param sqlConnection Established connection with database.
     * @param query         SQL Query for check.
     *
     * @return true if exists some row that verify the SQL query.
     */
    public static boolean dbCheckStatus(Connection sqlConnection, String query) {
        // Declarations 
        Statement statement;
        ResultSet result;
        boolean status;

        // Initialization
        status = false;
        try {
            statement = sqlConnection.createStatement();
            result = statement.executeQuery(query);
            // Get the number of manual modified status
            result.last();
            int numStatus = result.getRow();
            if (numStatus > 0) {
                status = true;
            }
        } catch (SQLException ex) {
            logger.warn("Error checking the status in the database!!" + ex.getMessage());
        }
        return status;
    }

    /**
     * dbStoreTemperature: store the temperature values in the database.
     *
     * I store the average of COUNTER_OF_TEMPERATURE values.
     *
     * @param client        Object client with all information.
     * @param sqlConnection Established connection with database.
     * @param t1            Solar temperature.
     * @param t2            External temperature.
     */
    public static void dbStoreTemperature(Client client, Connection sqlConnection, String t1, String t2) {
        int count;
        double temp1;
        double temp2;
        count = client.getCountTemp();
        count++;
        DecimalFormat decimalFormat = new DecimalFormat("00.00");

        temp1 = Double.parseDouble(t1);
        temp2 = Double.parseDouble(t2);
        if (count == client.getConfigTemperatureCounter()) {
            // Calculate the average of temperatures
            temp1 = (client.getSumTempSolar() + temp1) / count;
            temp2 = (client.getSumTempExt() + temp2) / count;
            try {
                // We store the temperature values in the temperature table
                PreparedStatement pstmt = sqlConnection.prepareStatement(Constants.SQL_TEMPERATURE_INSERT);
                pstmt.setString(1, decimalFormat.format(temp1));
                pstmt.setString(2, decimalFormat.format(temp2));
                pstmt.executeUpdate();
            } catch (SQLException ex) {
                logger.warn("Error storing the status in the database!!" + ex.getMessage());
            }
            // Init counter and sums
            client.setCountTemp(0);
            client.setSumTempSolar(0);
            client.setSumTempExt(0);
        } else {
            client.incCountTemp();
            client.addTempSolar(temp1);
            client.addTempExt(temp2);
        }
    }

    /**
     * dbDisableManualOrder: disable the specified manual order.
     *
     * @param sqlConnection Established connection with database.
     * @param order         Object with the manual order.
     */
    public static void dbDisableManualOrder(Connection sqlConnection, ManualOrder order) {
        PreparedStatement pstmt;
        ResultSet result;
        int id; // Id of 

        id = 0;
        try {
            // We store the temperature values in the temperature table
            pstmt = sqlConnection.prepareStatement(Constants.SQL_MANUAL_ORDERS_SEARCH);
            pstmt.setString(1, order.getServer());
            pstmt.setString(2, order.getDevice());
            pstmt.setString(3, order.getType());
            result = pstmt.executeQuery();
            while (result.next()) {
                id = result.getInt(1);
            }
            pstmt = sqlConnection.prepareStatement(Constants.SQL_MANUAL_ORDERS_UPDATE);
            pstmt.setTimestamp(1, getTimeStamp()); // Set update timestamp
            pstmt.setInt(2, id);
            pstmt.execute();
        } catch (SQLException ex) {
            logger.warn("Error searching manual orders!!" + ex.getMessage());
        }
    }

    /**
     * dbCleanStatus: Clean the status truncating the database.
     *
     * @param sqlConnection Established connection with database.
     */
    public static void dbCleanStatus(Connection sqlConnection) {
        Statement stmt;
        try {
            stmt = sqlConnection.createStatement();
            stmt.executeUpdate(Constants.SQL_STATUS_CLEAN);
        } catch (SQLException ex) {
            logger.warn("Error cleaning status on database!!" + ex.getMessage());
        }
    }

    /**
     * getTimeStamp: This method return the current timeStamp in SQL format.
     *
     * @return The current java.sql.timestamp.
     */
    public static java.sql.Timestamp getTimeStamp() {
        Calendar calendar = Calendar.getInstance();
        Date now = calendar.getTime();
        java.sql.Timestamp cs;

        cs = new java.sql.Timestamp(now.getTime());
        return cs;
    }

}
