package picontrol;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

/**
 *
 * Server: Class who represent the logic part of a Raspberry Pi working as
 * server
 *
 *
 */
public class Server {

    /**
     * Server attributes
     */
    private String identifier; // Identifier name
    private String ipAddress; // IP Address
    private Integer port; // TCP connection port
    private boolean online; // Server online
    private Date lastConnDate; // Last connection date
    private String lastCommand; // Last command send or received
    private String lastResponse; // Last response send or received
    private final ArrayList<Device> listDevices;

    // Constructor
    /**
     *
     */
    public Server() {
        // List of devices
        listDevices = new ArrayList<>();
        // We are online
        online = true;
    }

    /**
     * getIdentifier: get the identifier (name) of the server.
     *
     * @return the identifier
     */
    public String getIdentifier() {
        return identifier;
    }

    /**
     * setIdentifier: set the identifier (name) of the server.
     *
     * @param identifier the identifier to set
     */
    public void setIdentifier(String identifier) {
        this.identifier = identifier;
    }

    /**
     * getIpAddress: get the IP address of the server.
     *
     * @return the ipAddress
     */
    public String getIpAddress() {
        return ipAddress;
    }

    /**
     * setIpAddress: set the IP address of the server.
     *
     * @param ipAddress the ipAddress to set
     */
    public void setIpAddress(String ipAddress) {
        this.ipAddress = ipAddress;
    }

    /**
     * getPort: get the port of the server.
     *
     * @return the port
     */
    public Integer getPort() {
        return port;
    }

    /**
     * setPort: set the port of the server.
     *
     * @param port the port to set
     */
    public void setPort(Integer port) {
        this.port = port;
    }

    /**
     * getOnline: get the status of the server.
     *
     * @return if the server is Online
     */
    public boolean getOnline() {
        return online;
    }

    /**
     * setOnline: set the status of the server.
     *
     * @param Online the dateOnline to set
     */
    public void setOnline(boolean Online) {
        this.online = Online;
    }

    /**
     * getLastConnDate: get the last connection Date.
     *
     * @return the lastConnDate
     */
    public Date getLastConnDate() {
        return lastConnDate;
    }

    /**
     * setLastConnDate: set the last connection Date.
     *
     * @param lastConnDate the lastConnDate to set
     */
    public void setLastConnDate(Date lastConnDate) {
        this.lastConnDate = lastConnDate;
    }

    /**
     * setLastConnDate: set the last connection Date.
     *
     */
    public void setLastConnDate() {
        Date date = Calendar.getInstance().getTime();
        this.lastConnDate = date;
    }

    /**
     * getLastCommand: return the last command received on the server
     *
     * @return the lastCommand
     */
    public String getLastCommand() {
        return lastCommand;
    }

    /**
     * setLastCommand: set the last command received on the server
     *
     * @param lastCommand the lastCommand to set
     */
    public void setLastCommand(String lastCommand) {
        this.lastCommand = lastCommand;
    }

    /**
     * getLastResponse: return the last command response sent by the server.
     *
     * @return the lastResponse
     */
    public String getLastResponse() {
        return lastResponse;
    }

    /**
     * setLastResponse: set the last command response sent by the server.
     *
     * @param lastResponse the lastResponse to set
     */
    public void setLastResponse(String lastResponse) {
        this.lastResponse = lastResponse;
    }

    /**
     * addDevice: This add a Device to the list of devices in the Server
     *
     * @param device to add
     */
    public void addDevice(Device device) {
        this.listDevices.add(device);
    }

    /**
     * updateDevice: update the device in the device list
     *
     * @param device to update
     */
    public void updateDevice(Device device) {
        Device dev;
        int numDevices;

        numDevices = this.listDevices.size();
        for (int i = 0; i < numDevices; i++) {
            dev = this.listDevices.get(i);
            if (dev.getName().toUpperCase().equals(device.getName().toUpperCase())) {
                this.listDevices.set(i, device);
                i = numDevices; // Exit
            }
        }
    }

    /**
     * getNumDevices: return the number of devices on this server
     *
     * @return the number of devices
     */
    public int getNumDevices() {
        return listDevices.size();
    }

    /**
     * getListDevices: return the list of devices
     *
     * @return the listDevices
     */
    public ArrayList<Device> getListDevices() {
        return listDevices;
    }

    /**
     * getDevice: returns the device in the position specified.
     *
     * @param pos position to get the device.
     *
     * @return Device in specified position.
     */
    public Device getDevice(int pos) {
        if (getNumDevices() < pos) {
            return null;
        }
        return listDevices.get(pos);
    }

    /**
     * getDevice: returns the device whose name matches with the parameter.
     *
     * @param name Name of the device to get
     *
     * @return the device with the specified name.
     */
    public Device getDevice(String name) {
        for (Device dev : listDevices) {
            if (dev.getName().toUpperCase().equals(name.toUpperCase())) {
                return dev;
            }
        }
        return null;
    }

    /**
     * toString:
     *
     * @return string with the server configuration
     */
    @Override
    public String toString() {
        StringBuilder str = new StringBuilder();
        str.append("Identifier:     ").append(getIdentifier()).append("\n");
        str.append("LAN Address:    ").append(getIpAddress()).append("\n");
        str.append("Port:           ").append(getPort()).append("\n");
        if (online) {
            str.append("Num Devices:    ").append(getNumDevices()).append("\n\n");
            for (int i = 0; i < getNumDevices(); i++) {
                str.append("Device ").append(i).append("\n");
                str.append(getDevice(i)).append("\n");
            }
        } else {
            str.append("Server offline!!").append("\n");
        }
        return str.toString();
    }

}
