package picontrol;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

/**
 * Status: class who represent the status global of the system.
 *
 * The Status are formed by a list of Processor (controllers or computers), each one of it have a list of Peripherals
 *
 */
public class Status {

    // Attributes
    private Date datetime; // Date & Time of update
    private int numProcessors; // Number of logical servers
    private ArrayList<Processor> ProcessorList;

    // Constructors
    /**
     * Status(): Constructor without parameters.
     *
     */
    public Status() {
        this.ProcessorList = new ArrayList<>();
    }

    /**
     * Status(): Constructor with parameters.
     *
     * @param date of the status
     * @param numProcessors number of logic servers in the system
     */
    public Status(Date date, int numProcessors) {
        this.datetime = date;
        this.numProcessors = numProcessors;
        this.ProcessorList = new ArrayList<>(numProcessors);
    }

    /**
     * updateStatusTime: update the date and time
     */
    public void updateStatusTime() {
        this.datetime = Calendar.getInstance().getTime();
    }

    /**
     * getDate: return the date of the status
     *
     * @return the date
     */
    public Date getDate() {
        return datetime;
    }

    /**
     * setDate: set the date of the status
     *
     * @param date the date to set
     */
    public void setDate(Date date) {
        this.datetime = date;
    }

    /**
     * getNumProcessors: return the number of processors in the status system
     *
     * @return the numProcessors
     */
    public int getNumProcessors() {
        return numProcessors;
    }

    /**
     * setNumProcessors: set the number of processors in the status system
     *
     * @param numProcessors the numProcessors to set
     */
    public void setNumProcessors(int numProcessors) {
        this.numProcessors = numProcessors;
    }

    /**
     * incNumProcessors: increments the number of processors in the status system
     */
    public void incNumProcessors() {
        this.numProcessors++;
    }

    /**
     * getProcessorList: return the array list with all the processors
     *
     * @return the ProcessorList
     */
    public ArrayList<Processor> getProcessorList() {
        return ProcessorList;
    }

    /**
     * setProcessorList: set the array list with all the processors
     *
     * @param ProcessorList the ProcessorList to set
     */
    public void setProcessorList(ArrayList<Processor> ProcessorList) {
        this.ProcessorList = ProcessorList;
    }

    /**
     * addProcessorList: add a processor to the processor list
     *
     * @param proc Processor to add
     */
    public void addProcessorList(Processor proc) {
        this.ProcessorList.add(proc);
    }

    /**
     * updateProcessorList: update the processor in specified position
     *
     * @param proc processor to update
     * @param pos position to update
     */
    public void updateProcessorList(Processor proc, int pos) {
        if (pos < ProcessorList.size()) {
            this.ProcessorList.set(pos, proc);
        }
    }

    /**
     * getProcessor: return the processor whose name matches the specified
     *
     * @param name of the processor to search
     *
     * @return the processor with the specified name.
     */
    public Processor getProcessor(String name) {
        for (Processor c : ProcessorList) {
            if (c.getName().equals(name)) {
                return c;
            }
        }
        return null;
    }

    /**
     * getProcessor: return the processor in the given position
     *
     * @param pos position to get the processor
     *
     * @return the processor (logic server) in the specified position on the list
     */
    public Processor getProcessor(int pos) {
        if (pos > ProcessorList.size()) {
            return null;
        } else {
            return ProcessorList.get(pos);
        }
    }

    /**
     * toString
     *
     * @return string with the status.
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("GLOBAL STATUS ON ").append(getDate()).append("\n");
        for (int i = 0; i < ProcessorList.size(); i++) {
            sb.append(getProcessor(i).toString());
        }
        return sb.toString();
    }
}
