package piserver;

import java.util.Date;
import java.util.TimerTask;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * AutonomousMode: class who implements the autonomous working mode.
 *
 * This mode starts when the server detect no connection with the client.
 *
 */
public class AutonomousMode extends TimerTask {

    // Declarations
    ServerStatus server;
    Raspberry rpi;

    // Logger for logs and messages
    static final Logger logger = LogManager.getLogger(piserver.Communication.class.getName());

    /**
     * AutonomousMode: This class implements the autonomous mode.
     *
     * @param rpi:    Hardware configuration for set all devices to their default value!!
     * @param server: Server to verify the autonomous mode
     */
    public AutonomousMode(Raspberry rpi, ServerStatus server) {
        this.rpi = rpi;
        this.server = server;
    }

    @Override
    public void run() {
        // Declarations
        Date currentDate;
        Date lastConnDate;
        Long diff;
        Status currentStatus;

        currentDate = new Date();
        lastConnDate = server.getLastConnDate();

        // Difference in miliseconds
        diff = currentDate.getTime() - lastConnDate.getTime();
        // If the waiting time has elapsed
        if (diff > server.getConfigTimerAutonomousWaitTime()) {
            logger.warn("Going to autonomous mode!!!");
            currentStatus = server.getCurrentStatus();
            // If don't exist the current status
            if (currentStatus == null) {
                logger.warn("Not stored status!! Setting all devices to their default value!!");
                setDefault(server, rpi);
            } else {
                logger.warn("Autonomous mode start!!!");
                // Process autonomous mode!! => Process all rules
                for (AutonomousRule rule : server.getListRules()) {
                    String strRule = rule.getRule();
                    logger.warn("Verifying rule " + strRule);
                    if (verifyRule(currentStatus, strRule)) {
                        logger.warn("Processing " + strRule + ":" + rule.getDeviceName() + " - " + rule.getValue());
                        applyRule(rule.getDeviceName(), rule.getValue());
                    }
                }
            }
            // Set the last connection date to now
            server.setLastConnDate();
        }
    }

    /**
     * getTemp: method who return the value of a sensor temperature from the status.
     *
     * @param status
     *
     * @return
     */
    private String getValue(Status status, String sensorName) {
        String value;
        int numPeriph, numProc;
        int i, j;

        value = "";
        numPeriph = status.getProcessorList().size();
        for (i = 0; i < numPeriph; i++) {
            // I can use: for (Processor proc: status.getProcessorList), but....
            Processor proc = status.getProcessor(i);
            numProc = proc.getlistPeripherals().size();
            for (j = 0; j < numProc; j++) {
                Peripheral perif = proc.getPeripheral(j);
                if (perif.getName().equals(sensorName)) {
                    value = perif.getValue();
                    // I use for(i), and I can end in advance in both loops!!
                    j = numPeriph;
                    i = numProc;
                }
            }
        }
        return value;
    }

    /**
     * setDefault: this method set all devices to their default value.
     *
     * @param server
     */
    private void setDefault(Server server, Raspberry rpi) {
        // Setting all devices to their value!!
        server.getListDevices().forEach((dev) -> {
            InputOutput.setDefault(this.rpi, dev);
        });
    }

    /**
     * verifyRule: This method verify a rule write in JavaScript. In this way, it's easy
     * write complex rules.
     *
     * @param status
     * @param strRule
     *
     * @return
     */
    private boolean verifyRule(Status status, String strRule) {
        // Declarations
        // Objects for evaluate conditional expressions write in JavaScript
        ScriptEngineManager factory;
        ScriptEngine engine;
        String strEvaluate; // String with all values replaced.
        Boolean result; // Result of the evaluation. 

        // Inicializations
        factory = new ScriptEngineManager();
        engine = factory.getEngineByName("JavaScript");

        logger.trace("Evaluating " + strRule);

        /*
         * In the rule, replace the name of the sensor by their values. Also, replace values ON, OFF by 
         * JavaScript string 'ON' and 'OFF', so we can evaluate.
         */
        strEvaluate = strRule.replaceAll(Constants.SENSOR_TEMP_SOLAR, getValue(status, Constants.SENSOR_TEMP_SOLAR)).
                replaceAll(Constants.SENSOR_TEMP_AUX, getValue(status, Constants.SENSOR_TEMP_AUX)).
                replaceAll(Constants.OPTIMAL_TEMPERATURE_STRING, "" + server.getOptimalTemp()).
                replaceAll("ON", "'ON'").replaceAll("OFF", "'OFF'");
        // Isn't beautiful, but works!!

        logger.trace("Evaluating " + strEvaluate);
        try {
            result = (Boolean) engine.eval(strEvaluate);
        } catch (ScriptException se) {
            logger.warn("Error verifying rule " + strRule);
            logger.warn(se.getStackTrace());
            result = false;
        }
        return result;
    }

    /**
     * applyRule: this method apply the value to device with name.
     * This is, activate o deactivate the GPIO port corresponding to the specified device.
     *
     * @param strDevice
     * @param strValue
     */
    private void applyRule(String strDevice, String strValue) {
        Device device;

        logger.trace("Aplying value " + strDevice + " to device " + strDevice);
        device = server.getDevice(strDevice);
        switch (strValue) {
            case "ON":
                InputOutput.activateGPIO(rpi, device);
                break;
            case "OFF":
                InputOutput.deactivateGPIO(rpi, device);
                break;
            default:
        }

    }
}
