package piserver;

/*
 * #%L
 * %%
 * Copyright (C) 2016
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */
import com.pi4j.io.i2c.I2CDevice;
import java.io.IOException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * I2CDisplay: This class is based in
 *
 *
 * Java interface to the "Adafruit 0.54 inch Alphanumeric FeatherWing Display".
 * A detailed description of this 4-character LED Display is given here:
 * <a href="https://www.adafruit.com/products/3130">See FeatherWing Display </a>
 *
 * @author Eric Eliason
 *
 * -----------------------------------------
 * I have modified it only the necessary to adapt to my use. I have
 * converted the class instance to method displayInit, and I have adapted all their methods to
 * methods who I can call from my methods
 *
 */
public final class I2CDisplay {

    static final Logger logger = LogManager.getLogger(piserver.I2CDisplay.class.getName());

    //fixed values for writing HT16K33 displayLED buffer
    private static final int nBytes = 17;
    private static final int offset = 0;

    /**
     * displayInit: Turn on the HT16K33 chip, turn on the displayLED, set the blink
     * rate, and finally set displayLED brightness.
     *
     * @param rpi        Object who represents the actual hardware.
     * @param blink      (HT16K33.BLINK_1HZ, HT16K33.BLINK_2HZ, BLINK_HALFHZ,
     *                   HT16K33.BLINK_OFF)
     * @param brightness (HT16K33.DUTY_01 thru HT16K33.DUTY_16)
     * @param ledName    Name of the LED Display
     *
     * @return a I2C Device who represent a Display controller.
     */
    public static I2CDevice displayInit(Raspberry rpi, HT16K33 blink, HT16K33 brightness, String ledName) {

        I2CDevice adafruitFeather; // I2CDevice who represent a Display controller.

        adafruitFeather = null;
        if (blink != HT16K33.BLINK_1HZ
                && blink != HT16K33.BLINK_2HZ
                && blink != HT16K33.BLINK_HALFHZ
                && blink != HT16K33.BLINK_OFF) {
            logger.fatal("*** ERROR *** Invalid blink rate specified");
            System.exit(-1);
        }

        if (brightness != HT16K33.DUTY_01 && brightness != HT16K33.DUTY_02 && brightness != HT16K33.DUTY_03
                && brightness != HT16K33.DUTY_04 && brightness != HT16K33.DUTY_05 && brightness != HT16K33.DUTY_06
                && brightness != HT16K33.DUTY_07 && brightness != HT16K33.DUTY_08 && brightness != HT16K33.DUTY_09
                && brightness != HT16K33.DUTY_10 && brightness != HT16K33.DUTY_11 && brightness != HT16K33.DUTY_12
                && brightness != HT16K33.DUTY_13 && brightness != HT16K33.DUTY_14 && brightness != HT16K33.DUTY_15
                && brightness != HT16K33.DUTY_16) {
            logger.fatal("*** ERROR *** Invalid LED Brightness specified");
            System.exit(-1);
        }

        try {
            //i2cBus = I2CFactory.getInstance(I2C_BUS);
            //adafruitFeather = i2cBus.getDevice(I2C_DEVICE);

            adafruitFeather = rpi.getI2CDeviceFromMap(ledName);
            //turn on oscillator wakes up the HT16K33 chip
            adafruitFeather.write((byte) (HT16K33.CMD_SYSTEM_SETUP.get() | HT16K33.OSCILLATOR_ON.get()));
            Thread.sleep(100);

            //turn on all output rows of the HT16K33 chip (even though were only using 8 of them)
            adafruitFeather.write((byte) (HT16K33.CMD_ROW_INT_SET.get()));

        } catch (IOException | InterruptedException e) {
            logger.error("*** Error *** failed to connect with I2C Bus or I2C Device");
            logger.error(e.getStackTrace());
        }

        //clear the displayLED
        displayClear(adafruitFeather);

        //set the blink rate
        setBlinkMode(adafruitFeather, blink);

        //set the brightness level
        setBrigtness(adafruitFeather, brightness);

        return adafruitFeather;
    }

    /**
     * Clear the LED displayLED buffer--the four characters are blank
     *
     * @param adafruitFeather I2C Device who represent a Display controller.
     */
    public static void displayClear(I2CDevice adafruitFeather) {
        //Used to zero out HT16K33 displayLED buffer
        byte[] zeroDisplay = new byte[]{HT16K33.CMD_DISPLAY_POINTER.get(), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        try {
            adafruitFeather.write(zeroDisplay, offset, nBytes);
        } catch (IOException e) {
            logger.error("*** Error *** failed to clear 4-character LED Display");
            logger.error(e.getStackTrace());
        }
    }

    /**
     * Turn on the LED displayLED
     *
     * @param adafruitFeather I2C Device who represent a Display controller.
     * @param blink           value.
     */
    public static void displayOn(I2CDevice adafruitFeather, HT16K33 blink) {
        //set the blink rate and turn on the LED displayLED
        try {
            adafruitFeather.write((byte) (HT16K33.CMD_DISPLAY_SETUP.get() | blink.get() | HT16K33.DISPLAY_ON.get()));
        } catch (IOException e) {
            logger.error("*** Error *** failed to turn on LED display");
            logger.error(e.getStackTrace());
        }
    }

    /**
     * Turn off the LED displayLED. The displayLED buffer is not cleared so if
     * you turn on the displayLED again then contents of the displayLED buffer
     * reappear .
     *
     * @param adafruitFeather I2C Device who represent a Display controller.
     * @param blink           Blink mode.
     */
    public void displayOff(I2CDevice adafruitFeather, HT16K33 blink) {
        //set the blink rate and turn on the LED displayLED
        try {
            adafruitFeather.write((byte) (HT16K33.CMD_DISPLAY_SETUP.get() | blink.get() | HT16K33.DISPLAY_OFF.get()));
        } catch (IOException e) {
            logger.error("*** ERROR *** failed to turn off LED display");
            logger.error(e.getStackTrace());
        }
    }

    /**
     * Turning on the oscillator has the effect of taking the HT16K33 out of
     * standby mode. Turned on it is possible to then command the LED
     * displayLED.
     *
     * @param adafruitFeather I2C Device who represent a Display controller.
     */
    public static void oscillatorOn(I2CDevice adafruitFeather) {
        //turn on oscillator wakes up the HT16K33 chip
        try {
            adafruitFeather.write((byte) (HT16K33.CMD_SYSTEM_SETUP.get() | HT16K33.OSCILLATOR_ON.get()));
            Thread.sleep(100);
        } catch (IOException | InterruptedException e) {
            logger.error("*** ERROR *** failed to turn on oscillator");
            logger.error(e.getStackTrace());
        }
    }

    /**
     * Turning off the oscillator has the effect of putting the HT16K33 into
     * standby mode. Turned off it is not possible to then command the LED
     * displayLED until it is turned back on again.
     *
     * @param adafruitFeather I2C Device who represent a Display controller.
     */
    public static void oscillatorOff(I2CDevice adafruitFeather) {
        try {
            adafruitFeather.write((byte) (HT16K33.CMD_SYSTEM_SETUP.get() | HT16K33.OSCILLATOR_OFF.get()));
            Thread.sleep(100);
        } catch (IOException | InterruptedException e) {
            logger.error("*** ERROR *** failed to turn off oscillator");
            logger.error(e.getStackTrace());
        }
    }

    /**
     * Set the blink mode on the LED displayLED and turn the displayLED on.
     *
     * @param adafruitFeather I2C Device who represent a Display controller.
     * @param blink           (HT16K33.BLINK_1HZ, HT16K33.BLINK_2HZ, BLINK_HALFHZ,
     *                        HT16K33.BLINK_OFF)
     */
    public static void setBlinkMode(I2CDevice adafruitFeather, HT16K33 blink) {
        HT16K33 blinkDefault;
        if (blink != HT16K33.BLINK_1HZ
                && blink != HT16K33.BLINK_2HZ
                && blink != HT16K33.BLINK_HALFHZ
                && blink != HT16K33.BLINK_OFF) {
            logger.fatal("*** ERROR *** Invalid blink rate specified");
            System.exit(-1);
        }
        blinkDefault = blink;
        //set the blink rate and turn on the LED displayLED
        try {
            adafruitFeather.write((byte) (HT16K33.CMD_DISPLAY_SETUP.get() | blinkDefault.get() | HT16K33.DISPLAY_ON.get()));
        } catch (IOException e) {
            logger.error("*** ERROR *** failed to set blink mode");
            logger.error(e.getStackTrace());
        }
    }

    /**
     * Set the brightness level of the LED displayLED.
     *
     * @param adafruitFeather I2C Device who represent a Display controller.
     * @param brightness      (HT16K33.DUTY_01 thru HT16K33.DUTY_16)
     */
    public static void setBrigtness(I2CDevice adafruitFeather, HT16K33 brightness) {
        if (brightness != HT16K33.DUTY_01 && brightness != HT16K33.DUTY_02 && brightness != HT16K33.DUTY_03
                && brightness != HT16K33.DUTY_04 && brightness != HT16K33.DUTY_05 && brightness != HT16K33.DUTY_06
                && brightness != HT16K33.DUTY_07 && brightness != HT16K33.DUTY_08 && brightness != HT16K33.DUTY_09
                && brightness != HT16K33.DUTY_10 && brightness != HT16K33.DUTY_11 && brightness != HT16K33.DUTY_12
                && brightness != HT16K33.DUTY_13 && brightness != HT16K33.DUTY_14 && brightness != HT16K33.DUTY_15
                && brightness != HT16K33.DUTY_16) {
            logger.fatal("*** ERROR *** Invalid LED Brightness specified");
            System.exit(-1);
        }
        //set the brightness level of the LED displayLED
        try {
            adafruitFeather.write((byte) (HT16K33.CMD_BRIGHTNESS_SET.get() | brightness.get()));
        } catch (IOException e) {
            logger.error("*** ERROR *** failed to set display brigtness");
            logger.error(e.getStackTrace());
        }

    }

    /**
     * Display the passed four-character string on the LED Display
     *
     * -- Adapted for my displays, a Lucky Light KW4-56NCWB
     *
     * @param adafruitFeather I2C Device who represent a Display controller.
     * @param displayString   - must be a 4 character string.
     */
    public static void displayLED(I2CDevice adafruitFeather, String displayString) {
        /*
	 * This char array provides the bit-font for most alphanumeric characters
	 * in the ASCII table. The font array was extracted from the Adafruit_LED_Backpack
	 * software available here: https://github.com/adafruit/Adafruit_LED_Backpack
	 * 
         */
        final char[] ledFont = new char[]{
            0b0000000000000001,
            0b0000000000000010,
            0b0000000000000100,
            0b0000000000001000,
            0b0000000000010000,
            0b0000000000100000,
            0b0000000001000000,
            0b0000000010000000,
            0b0000000100000000,
            0b0000001000000000,
            0b0000010000000000,
            0b0000100000000000,
            0b0001000000000000,
            0b0010000000000000,
            0b0100000000000000,
            0b1000000000000000,
            0b0000000000000000,
            0b0000000000000000,
            0b0000000000000000,
            0b0000000000000000,
            0b0000000000000000,
            0b0000000000000000,
            0b0000000000000000,
            0b0000000000000000,
            0b0001001011001001,
            0b0001010111000000,
            0b0001001011111001,
            0b0000000011100011,
            0b0000010100110000,
            0b0001001011001000,
            0b0011101000000000,
            0b0001011100000000,
            0b0000000000000000, //  
            0b0000000000000110, // !
            0b0000001000100000, // "
            0b0001001011001110, // #
            0b0001001011101101, // $
            0b0000110000100100, // %
            0b0010001101011101, // &
            0b0000010000000000, // '
            0b0010010000000000, // (
            0b0000100100000000, // )
            0b0011111111000000, // *
            0b0001001011000000, // +
            0b0000100000000000, // ,
            0b0000000011000000, // -
            0b0000000010000001, // .
            0b0000110000000000, // /
            0b0000000000111111, // 0
            0b0000000000000110, // 1
            0b0000000001011011, // 2
            0b0000000001001111, // 3
            0b0000000001100110, // 4
            0b0000000001101101, // 5
            0b0000000001111101, // 6
            0b0000000000000111, // 7
            0b0000000001111111, // 8
            0b0000000001101111, // 9
            0b0001001000000000, // :
            0b0000101000000000, // ;
            0b0010010000000000, // <
            0b0000000011001000, // =
            0b0000100100000000, // >
            0b0001000010000011, // ?
            0b0000001010111011, // @
            0b0000000011110111, // A
            0b0001001010001111, // B
            0b0000000000111001, // C
            0b0001001000001111, // D
            0b0000000011111001, // E
            0b0000000001110001, // F
            0b0000000010111101, // G
            0b0000000011110110, // H
            0b0001001000000000, // I
            0b0000000000011110, // J
            0b0010010001110000, // K
            0b0000000000111000, // L
            0b0000010100110110, // M
            0b0010000100110110, // N
            0b0000000000111111, // O
            0b0000000011110011, // P
            0b0010000000111111, // Q
            0b0010000011110011, // R
            0b0000000011101101, // S
            0b0001001000000001, // T
            0b0000000000111110, // U
            0b0000110000110000, // V
            0b0010100000110110, // W
            0b0010110100000000, // X
            0b0001010100000000, // Y
            0b0000110000001001, // Z
            0b0000000000111001, // [
            0b0010000100000000, // 
            0b0000000000001111, // ]
            0b0000110000000011, // ^
            0b0000000000001000, // _
            0b0000000100000000, // `
            0b0001000001011000, // a
            0b0010000001111000, // b
            0b0000000011011000, // c
            0b0000100010001110, // d
            0b0000100001011000, // e
            0b0000000001110001, // f
            0b0000010010001110, // g
            0b0001000001110000, // h
            0b0001000000000000, // i
            0b0000000000001110, // j
            0b0011011000000000, // k
            0b0000000000110000, // l
            0b0001000011010100, // m
            0b0001000001010000, // n
            0b0000000011011100, // o
            0b0000000101110000, // p
            0b0000010010000110, // q
            0b0000000001010000, // r
            0b0010000010001000, // s
            0b0000000001111000, // t
            0b0000000000011100, // u
            0b0010000000000100, // v
            0b0010100000010100, // w
            0b0010100011000000, // x
            0b0010000000001100, // y
            0b0000100001001000, // z
            0b0000100101001001, // {
            0b0001001000000000, // |
            0b0010010010001001, // }
            0b0000010100100000, // ~
            0b0011111111111111};
        /*
         * displayBuffer contents are
	 * [0]    = the command for transferring 16-bytes to HT16K33 row pins (CMD_DISPLAY_POINTER)
	 * [1-8]  = bit font configuration for the 4-character displayLED (2 bytes per digit)
	 * [9-16] = always 0 because Adafruit feather wing only uses first 8.
         */
        byte[] displayBuffer = new byte[17];

        //index to the displayLED buffer
        int displayIndex = 0;

        int i;

        // The passed displayString must be exactly 4 characters in length.		 
        /*
         * Modified for use with my Display of 4+1 digit: 88:88
         *
         */
        if (displayString.length() != 5) {
            logger.error("*** Error *** The displayString passed must be exactly 5 characters!");
            logger.error("displayString " + displayString + " length found: " + displayString.length());
            // System.exit(-1);
            displayString = Utilities.left(displayString, 5);
        }

        // 1st byte is the displayLED command
        displayBuffer[displayIndex++] = HT16K33.CMD_DISPLAY_POINTER.get();

        //The next bytes define the bit font for the characters
        char[] digits = displayString.toCharArray();
        char digit;

        // Check if they exist a decimal point
        for (i = 0; i < digits.length - 1; i++) {
            digit = digits[i];
            char nextDigit = digits[i + 1];
            if (nextDigit != '.') {
                displayBuffer[displayIndex++] = (byte) (ledFont[digit]);
                displayBuffer[displayIndex++] = (byte) (ledFont[digit] >> 8);
            } else {
                byte value = (byte) (ledFont[digit]);
                value = (byte) (value | 128); // Set to 1 the MSB (128 = 10000000), used as decimal point
                displayBuffer[displayIndex++] = (byte) value;
                displayBuffer[displayIndex++] = (byte) (ledFont[digit] >> 8);
            }
        }
        // Now process the last digit
        digit = digits[i];
        if (digit == '.') {
            byte value = (byte) (ledFont[digit]);
            value = (byte) (value | 128); // Set to 1 the MSB (128 = 10000000), used as decimal point
            displayBuffer[displayIndex++] = (byte) value;
            displayBuffer[displayIndex++] = (byte) (ledFont[digit] >> 8);
        } else {
            displayBuffer[displayIndex++] = (byte) (ledFont[digit]);
            displayBuffer[displayIndex++] = (byte) (ledFont[digit] >> 8);
        }
        //fill remaining bytes with zeros
        for (i = displayIndex; i < nBytes; i++) {
            displayBuffer[i] = 0;
        }

        //display the 5 characters
        try {
            adafruitFeather.write(displayBuffer, offset, nBytes);
        } catch (IOException e) {
            logger.error("*** Error *** trying to write to HT16K33 Display buffer");
            logger.error(e.getStackTrace());
        }
    }

    /**
     * enum HT16K33 defines the commanding and parameter options for the HOLTEK
     * HT16K33 chip. RAM Mapping 16*8 LED Controller Driver and Keyscan. For
     * detailed information see:
     * <a href="https://cdn-shop.adafruit.com/datasheets/ht16K33v110.pdf">See
     * HT16K33</a>
     *
     * @author Eric ELiason
     */
    public enum HT16K33 {
        //Commands
        CMD_DISPLAY_POINTER((byte) 0X00),
        CMD_SYSTEM_SETUP((byte) 0X20),
        CMD_KEY_DATA_POINTER((byte) 0X40),
        CMD_INT_FLAG_POINTER((byte) 0X60),
        CMD_DISPLAY_SETUP((byte) 0X80),
        CMD_ROW_INT_SET((byte) 0XA0),
        CMD_BRIGHTNESS_SET((byte) 0XE0),
        //Parameters for CMD_SYSTEM_SETUP
        OSCILLATOR_OFF((byte) 0X00),
        OSCILLATOR_ON((byte) 0X01),
        //Parameters for CMD_DISPLAY_SETUP
        DISPLAY_OFF((byte) 0X00),
        DISPLAY_ON((byte) 0X01),
        BLINK_OFF((byte) 0X00),
        BLINK_2HZ((byte) 0X02),
        BLINK_1HZ((byte) 0X04),
        BLINK_HALFHZ((byte) 0X06),
        //Parameters for CMD_DIMMING_SET, specifies amount of LED dimming 
        DUTY_01((byte) 0X00),
        DUTY_02((byte) 0X01),
        DUTY_03((byte) 0X02),
        DUTY_04((byte) 0X03),
        DUTY_05((byte) 0X04),
        DUTY_06((byte) 0X05),
        DUTY_07((byte) 0X06),
        DUTY_08((byte) 0X07),
        DUTY_09((byte) 0X08),
        DUTY_10((byte) 0X09),
        DUTY_11((byte) 0X0A),
        DUTY_12((byte) 0X0B),
        DUTY_13((byte) 0X0C),
        DUTY_14((byte) 0X0D),
        DUTY_15((byte) 0X0E),
        DUTY_16((byte) 0X0F);

        private byte b;

        private HT16K33(byte b) {
            this.b = b;
        }

        public byte get() {
            return b;
        }
    }

    /**
     * testLCD: this method test a LED display counting from 0 to 9 on every position
     *
     * @param rpi Object who represents the actual hardware.
     * @param led String with the name of the led display to test.
     *
     */
    public static void testLCD(Raspberry rpi, String led) {
        String spc, lcd;
        I2CDevice adafruitFeather = rpi.getI2CDeviceFromMap(led);
        spc = "     ";
        // Testing LCD
        for (int j = 0; j < 5; j++) {
            for (int i = 0; i < 10; i++) {
                lcd = spc.substring(0, j) + i + spc.substring(j + 1);
                displayLED(adafruitFeather, lcd);
                try {
                    Thread.sleep(100);
                } catch (InterruptedException ex) {
                    logger.error(ex.getMessage());
                }
            }
        }
    }
}
