package piserver;

import com.pi4j.io.gpio.GpioPinDigitalMultipurpose;
import com.pi4j.io.gpio.Pin;
import com.pi4j.io.gpio.PinMode;
import com.pi4j.io.gpio.RaspiPin;
import com.pi4j.io.i2c.I2CDevice;
import com.pi4j.system.SystemInfo;
import java.io.IOException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * InputOutput: this class implement some of the specific methods of the Raspberry Pi Hardware
 *
 *
 */
public final class InputOutput {

    // Logger for logs and messages
    static final Logger logger = LogManager.getLogger(piserver.InputOutput.class.getName());

    /**
     * getHWBoard: Get the type of Raspberry board
     *
     * @return String with the type of board
     */
    public static String getHWBoard() {
        StringBuilder sb = new StringBuilder();
        try {
            sb.append(SystemInfo.getBoardType().name());
        } catch (IOException | InterruptedException | UnsupportedOperationException ex) {
            logger.error("Error getting type of hardware " + ex.getMessage());
            sb.append("Unknown board or error reading board!!");
        }
        return sb.toString();
    }

    /**
     * activateGPIO: This method activate the GPIO port specified, according to their level activation
     *
     * @param rpi    Object who represents the actual hardware.
     * @param device Device with the logical value.
     */
    public static void activateGPIO(Raspberry rpi, Device device) {
        // Declarations
        GpioPinDigitalMultipurpose pin;
        String pinName;

        // Before activate a GPIO, check if it's not already actived.
        device = readGPIO(rpi, device);
        // Initialization
        pinName = device.getName();
        pin = rpi.getPinDigitalFromMap(pinName);

        if (device.getValue().equals("OFF")) {
            switch (device.getParameter5()) {
                case "HIGH":
                    // Device activated at level high. 
                    pin.high();
                    break;
                case "LOW":
                    // Device activated at level low.
                    pin.low();
                    break;
            }
            logger.trace("Activated " + device.getName());
        } else {
            logger.trace("Device " + device.getName() + " already actived!!");
        }
    }

    /**
     * deactivateGPIO: This method deactivate the GPIO port specified, according to their level activation and de-activation.
     *
     * @param rpi    Object who represents the actual hardware.
     * @param device Device with the logical value.
     */
    public static void deactivateGPIO(Raspberry rpi, Device device) {
        //GpioController gpio;
        GpioPinDigitalMultipurpose pin;
        String pinName;

        // Before deactivate a GPIO, check if it is not already deactived
        device = readGPIO(rpi, device);
        pinName = device.getName();
        pin = rpi.getPinDigitalFromMap(pinName);
        if (device.getValue().equals("ON")) {
            switch (device.getParameter5()) {
                case "HIGH":
                    pin.low();
                    break;
                case "LOW":
                    pin.high();
                    break;
            }
            logger.trace("Deactivated " + device.getName());
        } else {
            logger.trace("Device " + device.getName() + " already deactived!!");
        }
    }

    /**
     * readGPIO: This method read the GPIO port specified. It's return ON or OFF,
     * according to the specified level for this port.
     *
     * @param rpi    Object who represents the actual hardware.
     * @param device Device with the logical value.
     *
     * @return Device with the value of the GPIO port updated.
     */
    public static Device readGPIO(Raspberry rpi, Device device) {
        // Declarations
        GpioPinDigitalMultipurpose pin;
        String pinName;
        boolean status;

        // Initialization
        pinName = device.getName();
        pin = rpi.getPinDigitalFromMap(pinName);
        // Get the pin status 
        status = pin.isHigh();
        // Check the level of signal 
        switch (device.getParameter5()) {
            case "LOW":
                // Level LOW => 0 = LOW, 1 = HIGH
                if (status) {
                    device.setValue("OFF");
                } else {
                    device.setValue("ON");
                }
                break;
            case "HIGH":
                // Level HIGH => 0 = HIGH, 1 = LOW
                if (status) {
                    device.setValue("ON");
                } else {
                    device.setValue("OFF");
                }
                break;
        }
        logger.trace("Readed " + device.getName() + " - Value: " + device.getValue());
        return device;
    }

    /**
     * getPinPi4j: convert the number of pin (integer) to type Pi4J.Pin
     *
     * @param n: number of pin.
     *
     * @return the equivalent type Pi4J Pin
     */
    public static Pin getPinPi4j(int n) {
        Pin pin;

        pin = null;
        switch (n) {
            case 0:
                pin = RaspiPin.GPIO_00;
                break;
            case 1:
                pin = RaspiPin.GPIO_01;
                break;
            case 2:
                pin = RaspiPin.GPIO_02;
                break;
            case 3:
                pin = RaspiPin.GPIO_03;
                break;
            case 4:
                pin = RaspiPin.GPIO_04;
                break;
            case 5:
                pin = RaspiPin.GPIO_05;
                break;
            case 6:
                pin = RaspiPin.GPIO_06;
                break;
            case 7:
                pin = RaspiPin.GPIO_07;
                break;
            case 8:
                pin = RaspiPin.GPIO_08;
                break;
            case 9:
                pin = RaspiPin.GPIO_09;
                break;
            case 10:
                pin = RaspiPin.GPIO_10;
                break;
            case 11:
                pin = RaspiPin.GPIO_11;
                break;
            case 12:
                pin = RaspiPin.GPIO_12;
                break;
            case 13:
                pin = RaspiPin.GPIO_13;
                break;
            case 14:
                pin = RaspiPin.GPIO_14;
                break;
            case 15:
                pin = RaspiPin.GPIO_15;
                break;
            case 16:
                pin = RaspiPin.GPIO_16;
                break;
            case 17:
                pin = RaspiPin.GPIO_17;
                break;
            case 18:
                pin = RaspiPin.GPIO_18;
                break;
            case 19:
                pin = RaspiPin.GPIO_19;
                break;
            case 20:
                pin = RaspiPin.GPIO_20;
                break;
            case 21:
                pin = RaspiPin.GPIO_21;
                break;
            case 22:
                pin = RaspiPin.GPIO_22;
                break;
            case 23:
                pin = RaspiPin.GPIO_23;
                break;
            case 24:
                pin = RaspiPin.GPIO_24;
                break;
            case 25:
                pin = RaspiPin.GPIO_25;
                break;
            case 26:
                pin = RaspiPin.GPIO_26;
                break;
            case 27:
                pin = RaspiPin.GPIO_27;
                break;
            case 28:
                pin = RaspiPin.GPIO_28;
                break;
            case 29:
                pin = RaspiPin.GPIO_29;
                break;
            case 30:
                pin = RaspiPin.GPIO_30;
                break;
            case 31:
                pin = RaspiPin.GPIO_31;
                break;
        }
        return pin;
    }

    /**
     * readI2C: This method return the device with the data value read from the I2C device
     *
     * @param rpi    Object who represents the actual hardware.
     * @param device Device with the logical value.
     *
     * @return Device with the data value updated with the value read from the I2C device.
     */
    public static Device readI2C(Raspberry rpi, Device device) {
        // Declarations
        String typeDevice; // Type of device using I2C bus
        String nameDevice; // Name of device using I2C bus
        I2CDevice i2cDevice; // I2C phisical device
        String i2cConfig; // Configuration for I2C device
        String conversor; // Conversor used
        String value; // Readed value

        // Initializations
        typeDevice = device.getParameter3(); // ADS1115
        nameDevice = device.getName(); // Sensor T
        i2cConfig = device.getParameter4();
        conversor = device.getParameter6();

        i2cDevice = rpi.getI2CDeviceFromMap(nameDevice);

        logger.trace("Reading I2C device " + nameDevice);
        switch (typeDevice) {
            // Of moment, I only have a ADS1115 ADC
            case "ADS1115":
                // Process i2cdevice config
                value = Integer.toString(readADS1115(i2cDevice, i2cConfig));
                // Now we have the raw digital value, so now, we need convert?
                switch (conversor) {
                    case "NTC1":
                    case "NTC2":
                        // If it's one of the Thermistor, convert the value...
                        device.setValue(value);
                        // From digital to voltage
                        device = Processing.digital2Volt(device);
                        // From voltage to omhnium
                        device = Processing.volt2Omh(device);
                        // And finally, form omhnium to temperature
                        value = Processing.NTC2Temp(device);
                        break;
                    case "UNK":
                    default:
                        // In this case, I return the digital value read by the ADC
                        device.setValue(value);
                }
                break;
            case "HT16K33":
                // This device is a Led Display, and it's only for output
                value = "";
                break;
            default:
                logger.warn("Unrecognized I2C device " + typeDevice);
                value = "Error!!";
        }
        device.setValue(value);
        return device;
    }

    /*
     * *************************************************************************************************
     * The ADS1115 had three internal registers:
     *
     * Pointer Register Byte (Write Only) 8bits
     * 0 0 0 0 0 0 x x -> xx set the register address:
     * This register is a Byte pointer to one of the other two 16 bits registers
     *     0 -> Conversion register
     *     1 -> Config register
     *
     * Conversion Register (Read only) 16bits
     * D15..D0 (bit D15 serves as "Ready" flag, and it's set when data is ready, in power-down single shot mode).
     *
     * Config Register (Read/Write) 16 bits
     *  15      -> OS: Flag bit for single shot
     *  14-12   -> MUX2, MUX1, MUX0: Select the input: 100 ANC0; 101 ANC1; 110 ANC2; 111 ANC3
     *  11-09   -> PGA2, PGA1, PGA0: Internal Amp gain. Default to 010
     *  08      -> MODE: Operational mode of the ADS1115.
     *      0: Continuous conversion mode.
     *      1: Power-down single-shot mode (default). In this case, bit D15 serves as "Ready" flag, 
     *          and it's set when data is ready.
     *  07-05   -> DR2, DR1, DR0: Set data rate. Default to 100 for 128SPS
     *  04-00   -> COMP_MODE, COMP_POL, COMP_LAT, COMP_QUE1, COMP_QUE0: Bits 4-0 are comparator functions.
     *
     *
     **********************************************************************************************
     */
    /**
     * readADS1115: This method read the ADS1115, previously configured, with the options specified
     *
     * @param i2cDevice I2C Device
     * @param config    String with the configuration.
     *
     * @return Integer with the read value by the ADC.
     */
    public static int readADS1115(I2CDevice i2cDevice, String config) {
        // Declarations
        byte[] configADS; // Configuration bytes send to ADS1115
        int sizeConfig; // Number of bytes to send
        String[] parameters; // String with the configuracion bytes in hex or bin format

        parameters = config.split("-");
        sizeConfig = parameters.length;
        configADS = new byte[sizeConfig];

        logger.trace("Configurating I2C device ADS1115 ");

        for (int i = 0; i < sizeConfig; i++) {
            if (parameters[i].toLowerCase().contains("0x")) {
                configADS[i] = Utilities.hexStringToByte(parameters[i]);
            } else {
                //configADS[i]=(byte) Byte.parseByte(Utilities.right(parameters[i],8), 2);
                configADS[i] = Utilities.binStringToByte(parameters[i]);
            }
        }
        return readADS1115(i2cDevice, configADS[0], configADS[1]);
    }

    /**
     * readADS1115: This method read the ADS1115, previously configured, with the specified options
     *
     * @param i2cDevice  I2C Device
     * @param highConfig High byte with the configuration.
     * @param lowConfig  Low byte with the configuration.
     *
     * @return integer with the value reade for the ADC.
     */
    public static int readADS1115(I2CDevice i2cDevice, byte highConfig, byte lowConfig) {
        // Declarations
        byte[] configADS;
        int adcValue;
        byte[] data;

        // Initialization
        configADS = new byte[2]; // Bytes to send to register configurations of device
        configADS[0] = highConfig;
        configADS[1] = lowConfig;
        // Size of data read 
        data = new byte[2];
        // Valued readed
        adcValue = 0;
        try {
            // Config ADS device -> register 0x01
            // write(int address, byte[] buffer, int offset, int size);
            i2cDevice.write(Constants.ADS1115_CONFIG_REGISTER, configADS, 0, 2);
            // Wait before read....
            Thread.sleep(1000);
            // Read data from Conversion Register 10 times and get the media
            for (int i = 0; i < Constants.ADS1115_NUMBER_SAMPLES; i++) {
                int adcRaw;
                // Read I2C device -> Read register 0x00
                i2cDevice.read(Constants.ADS1115_CONVERSION_REGISTER, data, 0, 2);
                // Convert the two bytes to integer.
                adcRaw = ((data[0] & 0xFF) * 256) + (data[1] & 0xFF);
                // Although the ADS1115 works with 16bits, the highest bit (D15) are used as Ready bit.
                // So, we only have 15bits: 2¹⁵ = 32768 => 0..32767
                if (adcRaw > 32767) {
                    adcRaw -= 65535;
                }
                adcValue += adcRaw;
            }
            adcValue = adcValue / Constants.ADS1115_NUMBER_SAMPLES;
        } catch (IOException | InterruptedException ex) {
            logger.warn("Error reading I2C device ADS1115");
        }
        return adcValue;
    }

    /**
     * initAllDevices: This method initializes all the physical devices of the raspberry.
     *
     * If the device is a GPIO pin, it set the digital pin to the desired status Input/Output and set the default value (ON/OFF or
     * UNDEFINED) in function or the logic level of the connected device (HIGH or LOW) It also add a the device to the map of
     * devices in the RPi.
     *
     * If the device is a I2C device, it also initialise it.
     *
     * @param server Server with the logical values.
     * @param rpi    Object who represents the actual hardware.
     *
     * @return the Raspberry object with all devices initialized.
     */
    public static Raspberry initAllDevices(Raspberry rpi, Server server) {
        // Declarations
        String pinFriendlyName;     // Friendly Pin name 
        Pin pinNumber;              // Pi4J pin Number
        /*
         * Type of digital pin. I use PinDigitalMultipurpose instead of PinDigitalInput and PinDigitalOutput. 
         * In this way, I only have one list of PinDigital
         */
        GpioPinDigitalMultipurpose pin;

        // Processing the I2C devices
        // String with the address and the config register
        String i2cAddress;
        // I2C device type
        I2CDevice i2cDevice = null;
        logger.trace("Initializating all devices");
        for (Device device : server.getListDevices()) {
            pinFriendlyName = device.getName();
            switch (device.getMode()) {
                case "IN":
                case "INPUT":
                    logger.trace("Creating GPIO INPUT");
                    // Get the pin input number for Pi4J
                    pinNumber = InputOutput.getPinPi4j(Integer.parseInt(device.getParameter3()));
                    // and set the Pin as Multipurpouse and Input mode
                    pin = rpi.getGpioController().provisionDigitalMultipurposePin(pinNumber, PinMode.DIGITAL_INPUT);
                    // Add Pin to the map of Pins
                    rpi.addPinDigitalToMap(pinFriendlyName, pin);
                    break;
                case "OUT":
                case "OUTPUT":
                    logger.trace("Creating GPIO OUTPUT");
                    // Get the pin input number for Pi4J
                    pinNumber = InputOutput.getPinPi4j(Integer.parseInt(device.getParameter3()));
                    // and set the Pin as Multipurpouse and Output mode
                    pin = rpi.getGpioController().provisionDigitalMultipurposePin(pinNumber, PinMode.DIGITAL_OUTPUT);
                    // Set the output pins to the default value: ON or OFF in their Logic Level
                    switch (device.getParameter6()) {
                        case "ON":
                            // Level High? => 1 = +V
                            switch (device.getParameter5()) {
                                case "HIGH":
                                    // And state to On
                                    pin.high();
                                    break;
                                case "LOW":
                                    pin.low();
                                    break;
                                default:
                            }
                            break;
                        case "OFF":
                            // Level High? => 1 = +V
                            switch (device.getParameter5()) {
                                case "HIGH":
                                    pin.low();
                                    break;
                                case "LOW":
                                    pin.high();
                                    break;
                                default:
                            }
                            break;
                    }
                    // Add Pin to the map of Pins
                    rpi.addPinDigitalToMap(pinFriendlyName, pin);
                    break;
                case "I2C":
                    logger.trace("Creating device I2C");
                    // Get the I2C Bus address
                    i2cAddress = device.getParameter2();
                    try {
                        i2cDevice = rpi.getI2cBus().getDevice(Integer.decode(i2cAddress));
                        logger.trace("I2Device created on address " + i2cAddress);
                    } catch (IOException ex) {
                        logger.error("Error getting I2C device." + ex.getMessage());
                    }
                    // Add the I2C device to the device list
                    rpi.addI2CDeviceToMap(i2cDevice, device.getName());
                    break;
            }
        }
        return rpi;
    }

    /**
     * setDefault: this method set the GPIO port to the default value.
     *
     * @param device Device with the logical value.
     * @param rpi    Object who represents the actual hardware.
     *
     * @return The Device with the values set to default.
     */
    public static Device setDefault(Raspberry rpi, Device device) {
        logger.trace("Setting device " + device.getName() + " to default value!!");
        // Set the default value for this GPIO port
        switch (device.getParameter6()) {
            case "ON":
                InputOutput.activateGPIO(rpi, device);
                // Set the value on the status
                device.setValue("ON");
                break;
            case "OFF":
                InputOutput.deactivateGPIO(rpi, device);
                // Set the value on the status
                device.setValue("OFF");
                break;
            default:
            // Other modes. We don't change anything
        }
        return device;
    }
}
