package piserver;

import com.google.gson.Gson;
import com.google.gson.JsonParseException;
import com.pi4j.io.i2c.I2CFactory;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.charset.StandardCharsets;
import java.util.StringTokenizer;
import java.util.Timer;
import java.util.TimerTask;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * PiServer: this class implements a the server running on a Raspberry.
 *
 * This server wait for a connection from the client, get their request by http methods (Get and Post), and reply a response.
 *
 *
 */
public class PiServer {

    // Json object to serialize data to send in the requests & responses
    private static Gson jsonObj;
    // Objects wit the message request and response
    private static Response objResponse;
    private static Request objRequest;
    // This boolean variable control when the serveris finished
    private static boolean endServer = false;
    // Object Global Status who implements a server object and the global status
    // Server object (logical configuration of server)
    private static ServerStatus server;
    // Raspberry object (hardware configuration of server)
    private static Raspberry rpi;

    // Logger for logs and messages
    static final Logger logger = LogManager.getLogger(piserver.PiServer.class.getName());

    /*
     * *****************************************************************************************************
     *
     * Server based on: https://docs.oracle.com/javase/tutorial/networking/sockets/clientServer.html
     *
     *
     * ServerSocket is a java.net class that provides a system-independent implementation of the server side of a client/server
     * socket connection. The constructor for ServerSocket throws an exception if it can't listen on the specified port (for
     * example, the port is already being used)
     *
     */
    public static void main(String args[]) throws IOException {

        // Class used in the connection
        ServerSocket serverSocket;  // Socket in the server
        Socket clientSocket;        // Socket established with the client
        BufferedReader messageFromClient;   // For read the message from the client
        DataOutputStream messageToClient;   // For send the message to the client

        String readLine; // String used to read lines from the http stream
        String query; // String with the query

        // Prepare the autonomous mode
        Timer timerAutonomous;
        TimerTask timerAutonomousTask;

        // Initializing the logger
        logger.warn("Init PiServer........");

        // Initializations 
        // -------------------------------------------------------------------
        //
        // Instantiate the two objects who define this Raspberry Pi Server..
        // ... the logic server
        server = new ServerStatus();
        // ... and the hardware equipment
        try {
            rpi = new Raspberry();
        } catch (I2CFactory.UnsupportedBusNumberException ex) {
            logger.error(ex.getMessage());
        }
        // Now, read the logical configuration of the server 
        server = Utilities.getConfig(server);
        // and the rules for autonomous mode...
        server = Utilities.getRules(server);
        // And now, configure the hardware
        rpi = InputOutput.initAllDevices(rpi, server);
        Utilities.welcome(server);// This is for show the logical info on the console
        // Before start the server, update their status       
        server = Processing.updateStatus(server, rpi);

        // If this server is the one with the temperarture sensor then, 
        // read temperature every minute and show it in the LED Displays
        Device solarDevice;
        solarDevice = server.getDevice(Constants.SENSOR_TEMP_SOLAR);

        if (solarDevice == null) {
            logger.trace("This server isn't run any scheduled task for sensor temperatures.");
        } else {
            // For read and show temperatures every minute
            logger.trace("Running scheduled task on temperature sensors devices.");
            // Declaration
            Timer timerTemp;
            TimerTask timerTempTask;
            // Initilization
            timerTemp = new Timer();
            timerTempTask = new GetTempTask((Server) server, rpi);
            timerTemp.schedule(timerTempTask, server.getConfigTimerLEDWaitStart(), server.getConfigTimerLEDRunEvery());
            // Prepare the LCD
            for (String[] array : Constants.DISPLAY_LCD_NAMES) {
                String led = array[0];
                System.out.println("Initializating display LED of 7 segments: " + led);
                rpi.updateI2CDeviceToMap(I2CDisplay.displayInit(rpi, I2CDisplay.HT16K33.BLINK_OFF, I2CDisplay.HT16K33.DUTY_08, led), led);
                // I2CDisplay.testLCD(rpi, led);
            }
        }
        // Initialize the autonomous timer task
        timerAutonomous = new Timer();
        timerAutonomousTask = new AutonomousMode(rpi, server);
        timerAutonomous.schedule(timerAutonomousTask, server.getConfigTimerAutonomousWaitStart(), server.getConfigTimerAutonomousRunEvery());
        logger.trace("Running scheduled task for autonomous mode!!.");

        // Now, begin the server 
        try {
            // Instantiate socket on desired port
            serverSocket = new ServerSocket(server.getPort());
            // Repeat forever or end if server receive a order to exit
            while (!endServer) {
                try {
                    clientSocket = serverSocket.accept();
                    // Create input (from client) and output (to client) data streams
                    messageFromClient = new BufferedReader(new InputStreamReader(clientSocket.getInputStream(), StandardCharsets.UTF_8));
                    messageToClient = new DataOutputStream(clientSocket.getOutputStream());
                    // Read the message from client
                    readLine = messageFromClient.readLine();
                    // The header is the first line readed
                    String header = readLine;
                    // Process the http header
                    StringTokenizer tokenizer = new StringTokenizer(header);
                    String method = tokenizer.nextToken(); // String with the http method
                    query = tokenizer.nextToken();  // String with the query
                    String strResponse; // String with the response
                    // Initialization of object JSON
                    jsonObj = new Gson();
                    // Http method used
                    switch (method) {
                        case "GET":     // GET Method
                            // Remove the slash on the end of the URL
                            query = query.replaceAll("/", "").toUpperCase();
                            logger.trace("GET request " + query);
                            // Process the GET request
                            switch (query) {
                                case "ONLINE":
                                    /*
                                     * Only send reply. To determine if the server is online, the important is the 
                                     * HttpURLConnection.HTTP_OK, not the message
                                     */
                                    strResponse = "online";
                                    server.setLastCommand(query);
                                    break;
                                case "SHUTDOWN":
                                case "REBOOT":
                                    endServer = true;
                                    Processing.closeDevices(server, rpi);
                                    Processing.shutdown(query);
                                    strResponse = query + " in 30sg!!";
                                    // End the program and shutdown the computer
                                    break;
                                case "EXIT":
                                    endServer = true;
                                    strResponse = "Ending!!";
                                    // end the program
                                    break;
                                default:
                                    strResponse = "Unknown GET command";
                            } // Switch Query
                            // Reply the GET Request to the client
                            logger.trace("Send GET Response!!");
                            Communication.sendResponse(HttpURLConnection.HTTP_OK, strResponse, server.getIdentifier(), messageToClient);
                            break;
                        case "POST":    // POST method
                            logger.trace("POST request!!");
                            // Wait for the 'blank line" between header and data
                            while ((readLine = messageFromClient.readLine()).length() != 0) {
                            }
                            // Read POST payload data, as data are send after a 'blank line'.
                            StringBuilder payload = new StringBuilder();
                            while (messageFromClient.ready()) {
                                // Payload contains the Data: command and parameters
                                payload.append((char) messageFromClient.read());
                            }
                            // Processing command
                            try {
                                // Get the object Request
                                objRequest = jsonObj.fromJson(payload.toString(), Request.class);
                                objResponse = Processing.processPOSTCommand(server, rpi, objRequest);
                            } catch (JsonParseException jse) {
                                // If the JSON Object is malformed, then were a transmission error.
                                logger.warn("Transmission error. Malformed JSON!!"+jse.getMessage());
                                objResponse.setMessage("Malformed JSON!!");
                                objResponse.setErrorCode(Constants.CONNECTION_ERROR.IO_ERROR);
                            } 
                            strResponse = jsonObj.toJson(objResponse);
                            Communication.sendResponse(HttpURLConnection.HTTP_OK, strResponse, server.getIdentifier(), messageToClient);
                            break;
                    }
                } catch (IOException ioe) {
                    logger.warn("Error establishing connection with client!!\n" + ioe.getMessage());
                } catch (RuntimeException re) {
                    logger.warn("Error unexpected!!!\n" + re.getStackTrace());
                }
            } // End While
        } catch (IOException ioe) {
            logger.warn("Error starting Pi server on " + server.getIpAddress() + ":" + server.getPort());
            logger.warn(ioe.getMessage());
        }
        // Close the log
        logger.warn("Closing the server!!!");
        System.exit(0);
    }
}
