package piserver;

import com.google.gson.Gson;
import com.google.gson.JsonParseException;
import java.io.IOException;
import static java.lang.Math.log;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Processing: class who implements methods for do some process.
 *
 *
 */
public final class Processing {

    // Logger for logs and messages
    static final Logger logger = LogManager.getLogger(piserver.Processing.class.getName());

    /**
     * processPOSTCommand: this method process the command send by POST http method
     *
     * @param server     Server with the logical devices.
     * @param rpi        Object who represents the actual hardware.
     * @param objRequest Object with the request
     *
     * @return Object Response.
     */
    public static Response processPOSTCommand(ServerStatus server, Raspberry rpi, Request objRequest) {
        // Declarations
        // ---------------------------------------------------------------
        Gson jsonObj; // JSON Object
        Response objResponse; // Object with the response
        String strResponse; // String with the response
        String command; // Command & parameter to execute by the server
        String parameter;
        Device device;  // Device to apply the command

        // Initialization
        // ---------------------------------------------------------------
        // JSON Object
        jsonObj = new Gson();
        // By default, we suposse NO_ERROR 
        objResponse = new Response(Constants.CONNECTION_ERROR.NO_ERROR);
        logger.trace("Processing POST Command!!");
        if (objRequest == null) {
            strResponse = "Error in transmision!!!";
            objResponse.setErrorCode(Constants.CONNECTION_ERROR.POST_ERROR);
            logger.warn("Error in POST transmission!!");
        } else {
            // Get the command
            command = objRequest.getCommand().toUpperCase();
            // And the parameter
            parameter = objRequest.getParameters();
            // It's a valid parameter
            switch (command) {
                case "CONFIG":
                    // Return the inicial status of the server (Server object)
                    logger.trace("Returning configuration!!");
                    server.setLastCommand(command);
                    server.setLastResponse(command);
                    server.setLastConnDate();
                    // Prepare the reply
                    strResponse = jsonObj.toJson((Server) server);
                    break;
                case "STATUS":
                    // Update the Status 
                    logger.trace("Updating status");
                    server = updateStatus(server, rpi);
                    server.setLastCommand(command);
                    server.setLastResponse(command);
                    server.setLastConnDate();
                    // Prepare the reply
                    strResponse = jsonObj.toJson((Server) server);
                    break;
                case "READ":
                    logger.trace("Reading " + parameter);
                    // Get the Device to execute command
                    device = server.getDevice(parameter);
                    // Read the info about a port.
                    InputOutput.readGPIO(rpi, device);
                    server.setLastCommand(command);
                    server.setLastResponse(command);
                    server.setLastConnDate();
                    // Prepare the reply
                    strResponse = jsonObj.toJson((Server) server);
                    break;
                case "ACTIVATE":
                    // We activate a GPIO Port
                    logger.trace("Activating " + parameter);
                    // Get the Device to execute command
                    device = server.getDevice(parameter);
                    // Activate the port
                    InputOutput.activateGPIO(rpi, device);
                    // Set the value on the status
                    device.setValue("ON");
                    // Update server
                    server.setLastCommand(command);
                    server.setLastResponse(command);
                    server.setLastConnDate();
                    // Prepare the reply
                    strResponse = jsonObj.toJson((Server) server);
                    break;
                case "DEACTIVATE":
                    // We deactivate a GPIO Port
                    logger.trace("Deactivating " + parameter);
                    // Get the Device to execute command
                    device = server.getDevice(parameter);
                    InputOutput.deactivateGPIO(rpi, device);
                    // Set the value on the status
                    device.setValue("OFF");
                    server.setLastCommand(command);
                    server.setLastResponse(command);
                    server.setLastConnDate();
                    // Prepare the reply
                    strResponse = jsonObj.toJson((Server) server);
                    break;
                case "DEFAULT":
                    // Get the Device to execute command
                    device = server.getDevice(parameter);
                    InputOutput.setDefault(rpi, device);
                    server.setLastCommand(command);
                    server.setLastResponse(command);
                    server.setLastConnDate();
                    // Prepare the reply
                    strResponse = jsonObj.toJson((Server) server);
                    break;
                case "UPDATE":
                    // Update the global status in this server
                    updateServer(server, parameter);
                    strResponse = "Update OK!!";
                    break;
                default:
                    logger.trace("Unreconigzed command " + command);
                    strResponse = "Unreconigzed command!!";
                    objResponse.setErrorCode(Constants.CONNECTION_ERROR.COMMAND_ERROR);
            }
        }
        objResponse.setMessage(strResponse);
        return objResponse;
    }

    /**
     * updateStatus: this method update the server status.
     *
     * @param server Server with the logical devices.
     * @param rpi    Object who represents the actual hardware.
     *
     * @return Server with the updated status.
     */
    public static ServerStatus updateStatus(ServerStatus server, Raspberry rpi) {
        Device device;
        int i;

        logger.trace("Processing Update status");
        // process all sensors
        for (i = 0; i < server.getNumDevices(); i++) {
            device = server.getDevice(i);
            // For each device, update their value
            // Only read input devices!!
            switch (device.getMode()) {
                case "I2C":
                    // Get the data from I2C device
                    device = InputOutput.readI2C(rpi, device);
                    break;
                case "INPUT":
                case "IN":
                    // Read Input device status
                    device = InputOutput.readGPIO(rpi, device);
                    break;
                case "OUTPUT":
                case "OUT":
                    // The output devices are set on use. We ignore it!!
                    break;
                default:
                // Ignore other type of devices, as we don't have any!!
                }
            device.setLastCommand("UPDATE STATUS");
            device.setLastDate();
            server.setLastConnDate();
            server.updateDevice(device);
        }
        return server;
    }

    /**
     * updateServer: This method update the global status stored in this server.
     *
     * @param server    Server object to update with the global status.
     * @param strStatus String with the status serialized in JSON.
     *
     * @return The Server updated.
     */
    public static ServerStatus updateServer(ServerStatus server, String strStatus) {
        // Declaration
        Gson jsonObj;  // Object to serilize objects in JSON.
        Status status; // Status object.

        // 
        jsonObj = new Gson();
        // Get the status
        try {
            // Get the object status
            status = jsonObj.fromJson(strStatus, Status.class);
            // And update
            server.exchangeStatus(status);
        } catch (JsonParseException ex) {
            // If the JSON Object is malformed, then they existed a transmission error.
            logger.warn("Transmission error. Malformed Status JSON!!\n" + ex.getMessage());
            // Don't update status
        }
        return server;
    }

    /**
     * shutdown: this method exec a shutdown or a restart command.
     *
     * @param opt String with the option.
     */
    public static void shutdown(String opt) {
        String command;

        command = "sudo shutdown ";
        switch (opt) {
            case "SHUTDOWN":
                command += "-h 1";
                break;
            case "REBOOT":
                command += "-r 1";
                break;
        }
        execCommandLinux(command);
    }

    /**
     * execCommandLinux: this method exec linux command.
     *
     * @param command String with the command to exec.
     */
    public static void execCommandLinux(String command) {
        Runtime runtime;
        Process proc;

        runtime = Runtime.getRuntime();
        try {
            proc = runtime.exec(command);
            logger.warn(proc.toString());
        } catch (IOException ioe) {
            logger.warn("Error trying do " + command + ":" + ioe.getMessage());
        }
    }

    /**
     * digital2Volt: this method return the device passed as parameter, but with the data converted
     * from the digital value read to Volts.
     *
     * The calculation is based on the values specified for the ADC converter.
     *
     * @param device to get the voltage from the digital value readed for the ADC.
     *
     * @return the device with the value converted to voltage.
     */
    public static Device digital2Volt(Device device) {
        // Special config?
        int adcValue;
        double voltage;

        adcValue = Integer.parseInt(device.getValue());
        voltage = Constants.ADS1115_GAIN * adcValue / Constants.ADS1115_RESOLUTION;
        device.setValue(Double.toString(voltage));
        return device;
    }

    /**
     * volt2Omh: this method return the device passed as parameter, but with the data converted
     * from Volts to Ohmns.
     *
     * The calculation is based on the value of resistance placed serially.
     *
     * @param device to get the resistance from the voltage
     *
     * @return the device with the value converted from voltage to resistance.
     */
    public static Device volt2Omh(Device device) {
        // Special config?
        double voltage;
        double resistance;
        String ntc;

        ntc = device.getParameter6();
        // Get the serial resistance value
        voltage = Double.parseDouble(device.getValue());
        /*
         *        R1 * Vo
         * Rt = --------------
         *        Vcc - Vo
         */
        resistance = (getNTCParameterValue(ntc, "SERIAL") * voltage) / (Constants.PI_VOLTAGE - voltage);
        device.setValue(Double.toString(resistance));
        return device;
    }

    /**
     * NTC2Temp: this method return the temperature of the device passed as parameter.
     *
     * The calculation is based on the Steinhart-Hart simplified B equation.
     *
     * @param device Device to obtain the temperature.
     *
     * @return String with the temperature.
     */
    public static String NTC2Temp(Device device) {
        double resistance;
        double temperature;
        String ntc;

        resistance = Double.parseDouble(device.getValue());
        ntc = device.getParameter6();

        /*
         * I use the Steinhart-Hart simplified B equation to translate the NTC resistance to temperature, as I know
         * the Beta Constant and Ro
         *
         * 1/T = 1/To + 1/B LN (R/Ro) 
         *
         * In this equation, the temperature are calculated in º Kelvin
         */
        temperature = resistance / getNTCParameterValue(ntc, "RZP"); // (R/Ro)
        temperature = log(temperature); // Ln (R/Ro)
        temperature = temperature / getNTCParameterValue(ntc, "BETA"); // 1/B * Ln(R/Ro)
        temperature = (1 / (getNTCParameterValue(ntc, "TNOMINAL") + Constants.KELVIN_DEGREE)) + temperature; //  1/To + 1/B * Ln(R/Ro)
        temperature = 1 / temperature; // 1/T
        temperature = temperature - Constants.KELVIN_DEGREE;
        String temperatureRound = String.format("%.02f", temperature);
        return temperatureRound;
    }

    /**
     * getNTCParameterValue: return the value of one parameter from the NTC.
     * This value is get from the array of configuration
     *
     * @param ntc   NTC to get the characteristic parameter.
     * @param value Name of the characteristic parameter.
     *
     * @return the value of one of the characteristic parameters of the NTC
     */
    public static int getNTCParameterValue(String ntc, String value) {
        String parameterValue;
        int pos;

        switch (value.toUpperCase()) {
            case "SERIAL":
                // Value of the serial resistance used to calcule the resistance of NTC thermistor
                pos = 4;
                break;
            case "RZP":
                // Value of resistance Rated Zero Power at temperature nominal (in Ohm)
                pos = 1;
                break;
            case "TNOMINAL":
                // Value of temp nominal in ºC
                pos = 2;
                break;
            case "BETA":
                // Beta coefficient of the thermistor
                pos = 3;
                break;
            default:
                logger.warn("Incorrect value to get NTC value " + value);
                return 0;
        }

        parameterValue = null;
        for (String[] array : Constants.NTC_VALUES) {
            if (array[0].equals(ntc)) {
                parameterValue = array[pos];
            }
        }
        return Integer.parseInt(parameterValue);
    }

    /**
     * closeDevices: this method close all OUT devices of the Raspberry.
     *
     * @param server Server with the logical devices.
     * @param rpi    Object who represents the actual hardware.
     */
    public static void closeDevices(Server server, Raspberry rpi) {
        logger.trace("Closing all devices.");
        for (Device device : server.getListDevices()) {
            if (device.getMode().equals("OUT")) { //  Only if it's a out device
                device = InputOutput.setDefault(rpi, device);
            }
        }
    }
}
