package piserver;

import java.util.ArrayList;

/**
 * Processor: this class implements a logic component on the network.
 *
 */
public class Processor {

    // Attributes 
    private String name; // Identifier name
    private String ip; // IP address
    private String type; // Client or Server
    private boolean online; // Is online?
    private ArrayList<Peripheral> listPeripherals; // List of devices of this server

    /**
     * Processor: Constructor without parameters.
     */
    public Processor() {
        this.listPeripherals = new ArrayList<>();
    }

    /**
     * Processor: Constructor with parameters.
     *
     * @param name   of the logical server.
     * @param ip     IP address of the processor.
     * @param online Indicate if the server is online.
     */
    public Processor(String name, String ip, boolean online) {
        this.listPeripherals = new ArrayList<>();
        this.name = name;
        this.ip = ip;
        this.online = online;
    }

    /**
     * getName: return the name of the processor (Logical Server).
     *
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * setName: set the name of the processor (Logical Server).
     *
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * getIp: return the IP address of the processor (Logical Server).
     *
     * @return the IP address
     */
    public String getIp() {
        return ip;
    }

    /**
     * stIp: set the IP address of the processor (Logical Server).
     *
     * @param ip the IP address to set
     */
    public void setIp(String ip) {
        this.ip = ip;
    }

    /**
     * isOnline: return if the processor (Logical Server) is online.
     *
     * @return the online
     */
    public boolean isOnline() {
        return online;
    }

    /**
     * setline: mark a processor (Logical Server) online.
     *
     * @param online the online to set
     */
    public void setOnline(boolean online) {
        this.online = online;
    }

    /**
     * getlistPeripherals: return the list of peripherals of this processor.
     *
     * @return the listPeripherals
     */
    public ArrayList<Peripheral> getlistPeripherals() {
        return listPeripherals;
    }

    /**
     * setListPeripherals: set the list of peripherals of this processor.
     *
     * @param listPeripherals the listPeripherals to set
     */
    public void setListPeripherals(ArrayList<Peripheral> listPeripherals) {
        this.listPeripherals = listPeripherals;
    }

    /**
     * addPeripheral: add a peripheral to this processor.
     *
     * @param peripheral Peripheral to add to the list of peripherals.
     */
    public void addPeripheral(Peripheral peripheral) {
        this.listPeripherals.add(peripheral);
    }

    /**
     * updatePeripheral: update the peripheral in specified position.
     *
     * @param peripheral the peripheral to update in specified position.
     * @param pos Position to update.
     */
    public void updatePeripheral(Peripheral peripheral, int pos) {
        this.listPeripherals.set(pos, peripheral);
    }

    /**
     * getPeripheral: return the peripheral with specified name.
     *
     * @param name of the peripheral to search
     *
     * @return the peripheral with specified name.
     */
    public Peripheral getPeripheral(String name) {
        for (Peripheral p : listPeripherals) {
            if (p.getName().equals(name)) {
                return p;
            }
        }
        return null;
    }

    /**
     * getPeripheral: return the peripheral in specified position.
     *
     * @param pos position in the list to get the peripheral.
     *
     * @return the peripheral in specified position.
     */
    public Peripheral getPeripheral(int pos) {
        if (listPeripherals.size() > pos) {
            return this.listPeripherals.get(pos);
        } else {
            return null;
        }
    }

    /**
     * getType: return the type of processor.
     *
     * @return the type
     */
    public String getType() {
        return type;
    }

    /**
     * setType: set the type of processor.
     *
     * @param type * the type to set
     */
    public void setType(String type) {
        this.type = type;
    }

    /**
     * toString: return the string with the processor values.
     *
     * @return the string with the logic server configuration.
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        if (isOnline()) {
            sb.append("Processor -> ").append(getName()).append(", working as ").append(getType()).append(".\n");
            for (int i = 0; i < listPeripherals.size(); i++) {
                sb.append(getPeripheral(i).toString());
            }
            sb.append("---------------------\n");
        }
        return sb.toString();
    }
}
