package piserver;

import com.pi4j.io.gpio.GpioController;
import com.pi4j.io.gpio.GpioFactory;
import com.pi4j.io.gpio.GpioPinDigitalMultipurpose;
import com.pi4j.io.i2c.I2CBus;
import com.pi4j.io.i2c.I2CDevice;
import com.pi4j.io.i2c.I2CFactory;
import java.io.IOException;
import java.util.HashMap;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Raspberry: This class implements the hardware devices used in the server.
 *
 * Initially this class extend the Server class, but I have problems with
 * the JSON serialization with the I2C object. So finally, I put apart the hardware
 * from the server object.
 *
 */
//public class Raspberry extends Server {
public class Raspberry {

    // Logger for logs and messages
    static final Logger logger = LogManager.getLogger(piserver.Raspberry.class.getName());

    private String board; // Raspberry board type    
    // Class used for GPIO: Create the instance of a GPIO controller. Only one by Raspberry
    private final GpioController gpioController;
    // Class used for use the I2C Bus
    private I2CBus i2cBus;
    // HashMaps with the GPIO & I2C devices used in this Raspberry
    private HashMap<String, GpioPinDigitalMultipurpose> pinDigitalMap;
    private HashMap<String, I2CDevice> i2cMap;

    /**
     * Constructor
     *
     * @throws com.pi4j.io.i2c.I2CFactory.UnsupportedBusNumberException if the I2C bus has not been enabled.
     * @throws IOException                                              If it happens an Input/Output error
     */
    public Raspberry() throws I2CFactory.UnsupportedBusNumberException, IOException {
        // Get  the model of Raspberry
        board = InputOutput.getHWBoard();
        // And instantiate the GPIO & I2C controllers
        gpioController = GpioFactory.getInstance();
        logger.trace("GPIO Factory set!!!");
        // In all new Raspberry, the I2CBus is the BUS_1
        i2cBus = I2CFactory.getInstance(I2CBus.BUS_1);
        logger.trace("I2CFactory set!!! " + i2cBus);
        // And the Maps with the info about GPIO & I2C Devices
        pinDigitalMap = new HashMap<>();
        i2cMap = new HashMap<>();
    }

    /**
     * getGpioController: return the only GPIO controller in one Raspberry.
     *
     * @return the gpioController
     */
    public GpioController getGpioController() {
        return gpioController;
    }

    /**
     * getBoard: Return the model of Raspberry Pi Board.
     *
     * @return the board
     */
    public String getBoard() {
        return board;
    }

    /**
     * setBoard: Set the model board.
     *
     * @param board the board to set
     */
    public void setBoard(String board) {
        this.board = board;
    }

    /**
     * getI2CBus: Return the bus I2C instantiate on the Raspberry.
     *
     * @return the i2cBus
     *
     */
    public I2CBus getI2cBus() {
        return i2cBus;
    }

    /**
     * stI2CBus: store the I2C Bus.
     *
     * @param i2cBus the i2cBus to set
     *
     */
    public void setI2cBus(I2CBus i2cBus) {
        this.i2cBus = i2cBus;
    }

    /**
     * getPinDigitalMap: Return the HashMap with all the GPIO defined in this Raspberry.
     *
     * @return the pinDigitalMap
     */
    public HashMap<String, GpioPinDigitalMultipurpose> getPinDigitalMap() {
        return pinDigitalMap;
    }

    /**
     * setPinDigitalMap: Store the HashMap with all the GPIO defined in this Raspberry
     *
     * @param pinDigitalMap the pinDigitalMap to set
     */
    public void setPinDigitalMap(HashMap<String, GpioPinDigitalMultipurpose> pinDigitalMap) {
        this.pinDigitalMap = pinDigitalMap;
    }

    /**
     * addPinDigitalToMap: append a GPIO to the hashmap with all the GPIO defined in this Raspberry
     *
     * @param name Name of the GPIO Pin to set into the map.
     * @param pin  GPIO pin
     */
    public void addPinDigitalToMap(String name, GpioPinDigitalMultipurpose pin) {
        this.pinDigitalMap.put(name, pin);
    }

    /**
     * getPinDigitalFromMap: get the GPIO with the specified name from to the Hash Map with all the GPIO defined in this Raspberry.
     *
     * @param name Name of the GPIO Pin to get from the map.
     *
     * @return the GPIO Pin.
     */
    public GpioPinDigitalMultipurpose getPinDigitalFromMap(String name) {
        return this.pinDigitalMap.get(name);
    }

    /**
     * getI2CMap: return the Hash Map with all I2C devices defined in this Raspberry.
     *
     * @return the i2cMap
     *
     */
    public HashMap<String, I2CDevice> getI2CMap() {
        return i2cMap;
    }

    /**
     * setI2CMap: store the Hash Map with all I2C devices defined in this Raspberry.
     *
     * @param i2cMap the i2cMap to set
     *
     */
    public void setI2cMap(HashMap<String, I2CDevice> i2cMap) {
        this.i2cMap = i2cMap;
    }

    /**
     * addI2CDeviceToMap: append a I2C device to the Hash Map with all I2C devices defined in this Raspberry
     *
     * @param i2cDevice I2C Device
     * @param i2cName   Name of the I2C Device in the map.
     */
    public void addI2CDeviceToMap(I2CDevice i2cDevice, String i2cName) {
        this.i2cMap.put(i2cName, i2cDevice);
    }

    /**
     * updateI2CDeviceToMap: update a I2C device into the Hash Map. This do the same as addI2CDeviceToMap, but
     * had a more clear name. Being a hash map, if a I2C device with that name exists, it updates its value.
     * And if not, add it.
     *
     * @param i2cDevice I2C device to add or update.
     * @param i2cName   Name of the I2C device.
     */
    public void updateI2CDeviceToMap(I2CDevice i2cDevice, String i2cName) {
        this.i2cMap.put(i2cName, i2cDevice);
    }

    /**
     * getI2CDeviceFromMap: return the I2C device with the specified name from the hash map
     * with all I2C devices defined in this Raspberry
     *
     * @param i2cName Name of the I2C device
     *
     * @return I2CDevice with the specified name.
     */
    public I2CDevice getI2CDeviceFromMap(String i2cName) {
        return this.i2cMap.get(i2cName);
    }
}
