package piserver;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

/**
 *
 * Server: Class who represent the logic part of a Raspberry Pi working as server
 *
 *
 */
public class Server {

    /**
     * Server attributes
     */
    private String identifier; // Identifier name
    private String ipAddress; // IP Address
    private Integer port; // TCP connection port
    private boolean online; // Server online
    private Date lastConnDate; // Last connection date
    private String lastCommand; // Last command send or received
    private String lastResponse; // Last response send or received
    private final ArrayList<Device> listDevices;

    private int configTimerLEDWaitStart; // Time for wait before start the LED Display Timer.
    private int configTimerLEDRunEvery; // Time for wait before update the LED Display.
    private int configTimerAutonomousWaitStart; // Time for wait before the autonomous mode start.
    private int configTimerAutonomousRunEvery; // Time for wait before check if start autonomous mode.
    private int configTimerAutonomousWaitTime; // Difference of time without connection, before start autonomous mode.

    // Constructor
    public Server() {
        // List of devices
        listDevices = new ArrayList<>();
        // We are online
        online = true;
    }

    /**
     * getIdentifier: get the identifier (name) of the server.
     *
     * @return the identifier
     */
    public String getIdentifier() {
        return identifier;
    }

    /**
     * setIdentifier: set the identifier (name) of the server.
     *
     * @param identifier the identifier to set
     */
    public void setIdentifier(String identifier) {
        this.identifier = identifier;
    }

    /**
     * getIpAddress: get the IP address of the server.
     *
     * @return the ipAddress
     */
    public String getIpAddress() {
        return ipAddress;
    }

    /**
     * setIpAddress: set the IP address of the server.
     *
     * @param ipAddress the ipAddress to set
     */
    public void setIpAddress(String ipAddress) {
        this.ipAddress = ipAddress;
    }

    /**
     * getPort: get the port of the server.
     *
     * @return the port
     */
    public Integer getPort() {
        return port;
    }

    /**
     * setPort: set the port of the server.
     *
     * @param port the port to set
     */
    public void setPort(Integer port) {
        this.port = port;
    }

    /**
     * getOnline: get the status of the server.
     *
     * @return if the server is Online
     */
    public boolean getOnline() {
        return online;
    }

    /**
     * setOnline: set the status of the server.
     *
     * @param Online the dateOnline to set
     */
    public void setOnline(boolean Online) {
        this.online = Online;
    }

    /**
     * getLastConnDate: get the last connection Date.
     *
     * @return the lastConnDate
     */
    public Date getLastConnDate() {
        return lastConnDate;
    }

    /**
     * setLastConnDate: set the last connection Date.
     *
     * @param lastConnDate the lastConnDate to set
     */
    public void setLastConnDate(Date lastConnDate) {
        this.lastConnDate = lastConnDate;
    }

    /**
     * setLastConnDate: set the last connection Date.
     *
     */
    public void setLastConnDate() {
        Date date = Calendar.getInstance().getTime();
        this.lastConnDate = date;
    }

    /**
     * getLastCommand: return the last command received on the server
     *
     * @return the lastCommand
     */
    public String getLastCommand() {
        return lastCommand;
    }

    /**
     * setLastCommand: set the last command received on the server
     *
     * @param lastCommand the lastCommand to set
     */
    public void setLastCommand(String lastCommand) {
        this.lastCommand = lastCommand;
    }

    /**
     * getLastResponse: return the last command response sent by the server.
     *
     * @return the lastResponse
     */
    public String getLastResponse() {
        return lastResponse;
    }

    /**
     * setLastResponse: set the last command response sent by the server.
     *
     * @param lastResponse the lastResponse to set
     */
    public void setLastResponse(String lastResponse) {
        this.lastResponse = lastResponse;
    }

    /**
     * addDevice: This add a Device to the list of devices in the Server
     *
     * @param device Device to add.
     */
    public void addDevice(Device device) {
        this.listDevices.add(device);
    }

    /**
     * updateDevice: update the device in the specified position.
     *
     * @param device Device to update.
     * @param pos    Position of the device to update.
     *
     * @return the position if the update was successful or -1 if not..
     */
    public int updateDevice(Device device, int pos) {
        int num = listDevices.size();
        if (pos >= num) {
            return -1;
        } else {
            this.listDevices.set(pos, device);
            return pos;
        }
    }

    /**
     * updateDevice: update the device in the device list, searching their position by their name.
     *
     * @param device to update.
     */
    public void updateDevice(Device device) {
        Device dev;
        int numDevices;

        numDevices = this.listDevices.size();
        for (int i = 0; i < numDevices; i++) {
            dev = this.listDevices.get(i);
            if (dev.getName().toUpperCase().equals(device.getName().toUpperCase())) {
                this.listDevices.set(i, device);
                i = numDevices; // Exit
            }
        }
    }

    /**
     * getNumDevices: return the number of devices on this server.
     *
     * @return the number of devices
     */
    public int getNumDevices() {
        return listDevices.size();
    }

    /**
     * getListDevices: return the list of devices.
     *
     * @return the listDevices
     */
    public ArrayList<Device> getListDevices() {
        return listDevices;
    }

    /**
     * getDevice: returns the device in the position specified.
     *
     * @param pos Position
     *
     * @return the device situated in the specified position.
     */
    public Device getDevice(int pos) {
        if (getNumDevices() < pos) {
            return null;
        }
        return listDevices.get(pos);
    }

    /**
     * getDevice: returns the device whose name matches with the parameter.
     *
     * @param name string with the name of the device to search.
     *
     * @return the device whose name matches with the parameter.
     */
    public Device getDevice(String name) {
        for (Device dev : listDevices) {
            if (dev.getName().toUpperCase().equals(name.toUpperCase())) {
                return dev;
            }
        }
        return null;
    }

    /**
     * getConfigTimerLEDWaitStart: get the time to wait before the LED Display start.
     *
     * @return the configTimerLEDWaitStart
     */
    public int getConfigTimerLEDWaitStart() {
        return configTimerLEDWaitStart;
    }

    /**
     * setConfigTimerLEDWaitStart: set the time to wait before the LED Display start.
     *
     * @param configTimerLEDWaitStart the configTimerLEDWaitStart to set
     */
    public void setConfigTimerLEDWaitStart(int configTimerLEDWaitStart) {
        this.configTimerLEDWaitStart = configTimerLEDWaitStart;
    }

    /**
     * getConfigTimerLEDRunEvery: get the value who indicates how long run the LED Display timer.
     *
     * @return the configTimerLEDRunEvery
     */
    public int getConfigTimerLEDRunEvery() {
        return configTimerLEDRunEvery;
    }

    /**
     * setConfigTimerLEDRunEvery: set the value who indicates how long run the LED Display timer.
     *
     * @param configTimerLEDRunEvery the configTimerLEDRunEvery to set
     */
    public void setConfigTimerLEDRunEvery(int configTimerLEDRunEvery) {
        this.configTimerLEDRunEvery = configTimerLEDRunEvery;
    }

    /**
     * getConfigTimerAutonomousWaitStart: get the time to wait before the Autonomous mode start.
     *
     * @return the configTimerAutonomousWaitStart
     */
    public int getConfigTimerAutonomousWaitStart() {
        return configTimerAutonomousWaitStart;
    }

    /**
     * setConfigTimerAutonomousWaitStart: set the time to wait before the Autonomous mode start.
     *
     * @param configTimerAutonomousWaitStart the configTimerAutonomousWaitStart to set
     */
    public void setConfigTimerAutonomousWaitStart(int configTimerAutonomousWaitStart) {
        this.configTimerAutonomousWaitStart = configTimerAutonomousWaitStart;
    }

    /**
     * getConfigTimerAutonomousRunEvery: get the value who indicates how long run the Autonomous mode timer.
     *
     * @return the configTimerAutonomousRunEvery
     */
    public int getConfigTimerAutonomousRunEvery() {
        return configTimerAutonomousRunEvery;
    }

    /**
     * setConfigTimerAutonomousRunEvery: set the value who indicates how long run the Autonomous mode timer.
     *
     * @param configTimerAutonomousRunEvery the configTimerAutonomousRunEvery to set
     */
    public void setConfigTimerAutonomousRunEvery(int configTimerAutonomousRunEvery) {
        this.configTimerAutonomousRunEvery = configTimerAutonomousRunEvery;
    }

    /**
     * getConfigTimerAutonomousWaitTime: get the difference of time without connection, before start autonomous mode.
     *
     * @return the configTimerAutonomousWaitTime
     */
    public int getConfigTimerAutonomousWaitTime() {
        return configTimerAutonomousWaitTime;
    }

    /**
     * setConfigTimerAutonomousWaitTime: Set the difference of time without connection, before start autonomous mode.
     *
     * @param configTimerAutonomousWaitTime the configTimerAutonomousWaitTime to set
     */
    public void setConfigTimerAutonomousWaitTime(int configTimerAutonomousWaitTime) {
        this.configTimerAutonomousWaitTime = configTimerAutonomousWaitTime;
    }

    /**
     * toString:
     *
     * @return the string with the server configuration
     */
    @Override
    public String toString() {
        StringBuilder str = new StringBuilder();
        str.append("Identifier:     ").append(getIdentifier()).append("\n");
        str.append("LAN Address:    ").append(getIpAddress()).append("\n");
        str.append("Port:           ").append(getPort()).append("\n");
        if (online) {
            str.append("Num Devices:    ").append(getNumDevices()).append("\n\n");
            for (int i = 0; i < getNumDevices(); i++) {
                str.append("Device ").append(i).append("\n");
                str.append(getDevice(i)).append("\n");
            }
        } else {
            str.append("Server offline!!").append("\n");
        }
        return str.toString();
    }

}
