package piserver;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/*
 * Utilidades
 * Clases utilizadas en el servidor
 */
// Nombre del paquete
/**
 *
 *
 */
// Constructor de la clase que representa a las utilidades
public class Utilities {

    // Logger for logs and messages
    static final Logger logger = LogManager.getLogger(piserver.Utilities.class.getName());

    /*
     * Limpiar mensaje: Eliminar espacios del principio y del fin del mensaje, y sustituir los espacios consecutivos por uno solo
     */
    public static String CleanString(String msg) {
        String cleaned = msg;
        try {
            cleaned = cleaned.replaceAll(" +", " ").trim();
        } catch (Exception e) {
            logger.fatal("Error on configuration file!! " + e.getMessage());
            System.exit(-1);
        }
        return cleaned;
    }

    /**
     * welcome: this method shows the welcome message
     *
     * @param server to show the welcome message.
     */
    public static void welcome(Server server) {
        // Welcome message
        System.out.println("  Server Solar Server  ");
        System.out.println("---------------------------");
        System.out.println("");

        System.out.println("");
        System.out.println(server.getIdentifier() + " waiting on " + server.getIpAddress() + ":" + server.getPort());
        System.out.println("");
        System.out.println(server);
        System.out.println("Num Devices:" + server.getNumDevices());
        System.out.println("-----------------------------------------\n");
    }

    /**
     * getConfig: this method read the configuration file.
     *
     * @param server Object Server with the status 
     *
     * @return Object serverStatus updated with the configuration 
     */
    public static ServerStatus getConfig(ServerStatus server) {
        // Declarations
        InputStream is;
        Properties properties;
        String filename;
        String aux;

        // Initialization
        properties = new Properties();
        is = null;

        logger.trace("Getting configuration file.");
        // Config File Name
        filename = Constants.CONFIG_PATH + Constants.SERVER_CONFIG_FILENAME;
        // Read config filename
        try {
            // Read properties and set server values
            is = new FileInputStream(filename);
            // Load properties file
            properties.load(is);
            server.setIdentifier(properties.getProperty("identifier"));
            server.setIpAddress(properties.getProperty("address"));
            server.setPort(Integer.parseInt(Utilities.CleanString(properties.getProperty("port"))));
            server.setOptimalTemp(Double.parseDouble(properties.getProperty("optimalTemperature")));
            server.setConfigTimerLEDWaitStart(Integer.parseInt(properties.getProperty("ledWaitStart")));
            server.setConfigTimerLEDRunEvery(Integer.parseInt(properties.getProperty("ledRunEvery")));
            server.setConfigTimerAutonomousWaitStart(Integer.parseInt(properties.getProperty("timerAutoWaitStart")));
            server.setConfigTimerAutonomousRunEvery(Integer.parseInt(properties.getProperty("timerAutoRunEvery")));
            server.setConfigTimerAutonomousWaitTime(Integer.parseInt(properties.getProperty("timerAutoWaitTime")));

            int numDevices = Integer.parseInt(Utilities.CleanString(properties.getProperty("numdevices")));
            logger.trace("Num Devices on config file " + numDevices);
            Device device;
            for (int i = 0; i < numDevices; i++) {
                device = new Device();
                aux = properties.getProperty("device" + (i + 1));
                String[] parameters = aux.split(",");
                device.setName(parameters[0]); // Sensor1
                device.setMode(CleanString(parameters[1].toUpperCase())); // IN/OUT/I2C
                /*
                 * Now, we save the physical pin number and the Pi4J pin number,
                 * or if it's a I2C device, we save the Hex Address of I2C Bus 
                 * and the name or type of I2C device */
                device.setParameter2(CleanString(parameters[2]));
                device.setParameter3(CleanString(parameters[3]));
                device.setParameter4(Utilities.CleanString(parameters[4])); // Pin logic name
                device.setParameter5(Utilities.CleanString(parameters[5])); // Logic level (LOW/HIGH)
                device.setParameter6(Utilities.CleanString(parameters[6])); // Default value
                device.setComment(parameters[7]); // Comment
                logger.trace("Adding device!!! " + i);
                server.addDevice(device);
            }
            is.close();
        } catch (FileNotFoundException fnf) {
            logger.fatal("Config file " + Constants.SERVER_CONFIG_FILENAME + " not found!!");
            System.exit(-1);
        } catch (IOException ioe) {
            logger.fatal("Input/Output Error " + ioe.getMessage());
            System.exit(-1);
        } catch (NumberFormatException e) {
            logger.fatal("Error parsing config file.\n" + e.getStackTrace());
            System.exit(-1);
        }
        return server;
    }

    /**
     * getRules: this method read the rules configuration file.
     *
     * @param server: server who store the rules.
     *
     * @return the server with the rules
     */
    public static ServerStatus getRules(ServerStatus server) {
        // Declarations
        InputStream is;
        Properties properties;
        String filename;
        String aux;
        AutonomousRule rule;

        // Initialization
        properties = new Properties();
        is = null;

        logger.trace("Getting rules configuration file.");
        // Config File Name
        filename = Constants.CONFIG_PATH + Constants.RULES_CONFIG_FILENAME;
        // Read config filename
        try {
            is = new FileInputStream(filename);
        } catch (FileNotFoundException fnf) {
            logger.fatal("Config file " + Constants.RULES_CONFIG_FILENAME + " not found!!");
            System.exit(-1);
        }
        try {
            // Load properties file
            properties.load(is);
            int numRules = Integer.parseInt(Utilities.CleanString(properties.getProperty("numRules")));
            logger.trace("Number of Rules on rules config file " + numRules);
            for (int i = 0; i < numRules; i++) {
                rule = new AutonomousRule();
                aux = properties.getProperty("rule" + (i + 1));
                String[] parameters = aux.split(",");
                logger.trace("Adding rule ! " + (i + 1));
                rule.setRule(parameters[0]);
                rule.setDeviceName(parameters[1]);
                rule.setValue(parameters[2]);
                server.addListRules(rule);
            }
            is.close();
            // Read properties and set server values
        } catch (FileNotFoundException fnfe) {
            logger.fatal("Error reading config rule file: " + Constants.RULES_CONFIG_FILENAME);
            logger.fatal(fnfe.getMessage());
            System.exit(-1);
        } catch (IOException ioe) {
            logger.fatal("Input/Output Error " + ioe.getMessage());
            System.exit(-1);
        } catch (NumberFormatException e) {
            logger.fatal("Error parsing config rule file.\n" + e.getStackTrace());
            System.exit(-1);
        }
        return server;
    }

    /**
     * hexStringToByte: this method convert a hexadecimal string in format 0xHH to byte
     *
     * I need convert first to integer because Java don't have unsigned bytes.
     *
     * https://sites.google.com/site/gencoreoperative/index/java-development/java-unsigned-bytes
     *
     * @param hexString: Hexadecimal string to convert to byte.
     *
     * @return byte value of the hexadecimal string.
     */
    public static byte hexStringToByte(String hexString) {
        int intValue; // Integer value
        hexString = right(hexString.trim(), 2); // Get only the 2 right chars
        intValue = Integer.parseInt(hexString, 16); // Convert to integer
        return (byte) intValue; // Return byte
    }

    /**
     * binStringToByte: this method convert a binary string in format 0bdddddddd to byte
     *
     * I need convert first to integer because Java don't have unsigned bytes.
     *
     * https://sites.google.com/site/gencoreoperative/index/java-development/java-unsigned-bytes
     *
     * @param binString: binary string to convert to byte.
     *
     * @return the byte value of the binary String.
     */
    public static byte binStringToByte(String binString) {
        int intValue; // Integer value
        binString = right(binString.trim(), 8); // Get only the 8 right chars
        intValue = Integer.parseInt(binString, 2); // Convert to integer
        return (byte) intValue; // Return byte
    }

    // Right$, Left$ and Mid$ functions to work with strings!!!
    /**
     * right: this method return the 'size' chars positioned on the right of the string
     *
     * @param str  String to get the n characters positioned on the right.
     * @param size Number of chars to get.
     *
     * @return the substring of string str, formed by size chars from the right.
     */
    public static String right(String str, int size) {
        return str.substring(str.length() - size);
    }

    /**
     * left: this method return the 'size' chars positioned on the left of the string
     *
     * @param str  String to get the n characters positioned on the left.
     * @param size Number of chars to get.
     *
     * @return the substring of string str, formed by size chars from the left.
     */
    public static String left(String str, int size) {
        return str.substring(0, size);
    }

    /**
     * mid: this method return the 'size' chars, from position 'pos'.
     *
     * @param str String to get the n characters positioned on the left.
     * @param pos Position to begin to get the chars.
     * @param size Number of chars to get.
     *
     * @return the string in the middle
     */
    public static String mid(String str, int pos, int size) {
        if ((pos > str.length()) || (pos + size > str.length())) {
            return str;
        } else {
            return str.substring(pos - 1, pos + size - 1);
        }
    }
}
